package ar.com.macro.adapter.controllers;

import javax.validation.Valid;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ar.com.macro.adapter.services.AltaAcuerdoCtaCteService;
import ar.com.macro.adapter.services.ProcesoService;
import ar.com.macro.adapter.services.TycsService;
import ar.com.macro.bpm.scope.AltaAcuerdoCtaCteScopeHelper;
import ar.com.macro.core.application.incoming.request.AltaAcuerdoCtaCteRequest;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.core.application.incoming.request.FirmaRequest;
import ar.com.macro.core.application.incoming.response.AltaAcuerdoCtaCteResponse;
import ar.com.macro.core.application.incoming.response.CancelacionResponse;
import ar.com.macro.core.application.incoming.response.CondicionesResponse;
import ar.com.macro.core.application.incoming.response.ConsultaTycsResponse;
import lombok.RequiredArgsConstructor;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/altaAcuerdoCtaCte")
public class AltaAcuerdoCtaCteController {

    private final AltaAcuerdoCtaCteService altaAcuerdoCtaCteService;
    
    private final TycsService tycsService;
    
    private final ProcesoService procesoService;

    @PostMapping("/iniciar")
    public ResponseEntity<AltaAcuerdoCtaCteResponse> iniciar(@Valid @RequestBody AltaAcuerdoCtaCteRequest altaAcuerdoEnCtaRequest) {
    	
        String processInstanceId = altaAcuerdoCtaCteService.iniciar(altaAcuerdoEnCtaRequest);

        AltaAcuerdoCtaCteResponse altaAcuerdoCtaCteResponse = AltaAcuerdoCtaCteResponse
        		.builder()
        		.resultado(HttpStatus.OK.name())
        		.codResultado(HttpStatus.OK.value())
        		.descripcion(HttpStatus.OK.series().name())
        		.idSolicitud(processInstanceId)
        		.estadoDelProceso(procesoService.getEstadoDelProceso(processInstanceId))
        		.build();
        
        return ResponseEntity.ok().body(altaAcuerdoCtaCteResponse);
    }
    
    @PostMapping("/firmaRealizada")
    public ResponseEntity<AltaAcuerdoCtaCteResponse> validarFirma(@Valid @RequestBody FirmaRequest fimarRequest) {

        String idSolicitud = altaAcuerdoCtaCteService.recibirFirma(fimarRequest, new AltaAcuerdoCtaCteScopeHelper(null));

        AltaAcuerdoCtaCteResponse altaAcuerdoCtaCteResponse = AltaAcuerdoCtaCteResponse
                .builder()
                .resultado(HttpStatus.OK.name())
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .idSolicitud(idSolicitud)
				.estadoDelProceso(procesoService.getEstadoDelProceso(fimarRequest.getIdSolicitud()))
                .build();

        return ResponseEntity.ok(altaAcuerdoCtaCteResponse);
    }
    
    @GetMapping("/consultaTycs/idSolicitud/{idSolicitud}")
    public ResponseEntity<ConsultaTycsResponse> consultaTycs(@PathVariable String idSolicitud) {

    	ConsultaTycsResponse consultaTycsResponse = ConsultaTycsResponse
                .builder()
                .resultado(HttpStatus.OK.name())
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .idSolicitud(idSolicitud)
                .listaIdsTycs(tycsService.getTycsPorIdSolicitud(idSolicitud))
                .build();
        
        return ResponseEntity.ok(consultaTycsResponse);
    }
    
    //TODO[EDPC-99-juliano] não temos ainda serviço no ESB, será endpoint ou delegate?
    @GetMapping("/condicionesAcuerdoCC/{canal}/{idProducto}")
    public ResponseEntity<CondicionesResponse> condicionesAcuerdoCC(@PathVariable String canal, @PathVariable String idProducto) {

        return ResponseEntity.ok().body(altaAcuerdoCtaCteService.buscarParametros(canal, idProducto));
    }
    
    @PostMapping("/cancelacion")
    public ResponseEntity<CancelacionResponse> cancelacion(@Valid @RequestBody CancelacionRequest cancelacionRequest) {

    	altaAcuerdoCtaCteService.cancelacion(cancelacionRequest, new AltaAcuerdoCtaCteScopeHelper(null));

        CancelacionResponse cancelacion = CancelacionResponse
                .builder()
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .build();

        return ResponseEntity.ok(cancelacion);
    }

}
