package ar.com.macro.adapter.controllers;

import java.util.List;

import javax.validation.Valid;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ar.com.macro.adapter.services.AltaComercioService;
import ar.com.macro.adapter.services.ProcesoService;
import ar.com.macro.adapter.services.TycsService;
import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.core.application.incoming.request.AltaComercioRequest;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.core.application.incoming.request.FirmaRequest;
import ar.com.macro.core.application.incoming.response.AltaComercioResponse;
import ar.com.macro.core.application.incoming.response.CancelacionResponse;
import ar.com.macro.core.application.incoming.response.ConsultaEstadoSolicitudResponse;
import ar.com.macro.core.application.incoming.response.ConsultaTycsResponse;
import lombok.RequiredArgsConstructor;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/altaComercio")
public class AltaComercioController {

    private final AltaComercioService altaComercioService;
    
    private final TycsService tycsService;
    
    private final ProcesoService procesoService;

    @PostMapping("/iniciar")
    public ResponseEntity<AltaComercioResponse> iniciar(@Valid @RequestBody AltaComercioRequest altaComercioRequest) {

        String processInstanceId = altaComercioService.iniciar(altaComercioRequest);

        AltaComercioResponse altaComercioResponse = AltaComercioResponse
                .builder()
                .resultado(HttpStatus.OK.name())
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .idSolicitud(processInstanceId)
                .estadoDelProceso(procesoService.getEstadoDelProceso(processInstanceId))
                .build();

        return ResponseEntity.ok().body(altaComercioResponse);
    }

    @PostMapping("/firmaRealizada")
    public ResponseEntity<AltaComercioResponse> validarFirma(@Valid @RequestBody FirmaRequest fimarRequest) {

        String idSolicitud = altaComercioService.recibirFirma(fimarRequest, new AltaComercioScopeHelper(null));

        AltaComercioResponse altaComercioResponse = AltaComercioResponse
                .builder()
                .resultado(HttpStatus.OK.name())
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .idSolicitud(idSolicitud)
                .estadoDelProceso(procesoService.getEstadoDelProceso(fimarRequest.getIdSolicitud()))
                .build();

        return ResponseEntity.ok(altaComercioResponse);
    }

    //TODO[EDPC-108-felipe] não temos ainda serviço no ESB
    @GetMapping("/consultaEstadoSolicitud")
    public ResponseEntity<List<ConsultaEstadoSolicitudResponse>> consultaEstadoSolicitud(
        @RequestParam String canal,
        @RequestParam String tipoDoc,
        @RequestParam Long nroTributario,
        @RequestParam(required = false) String idSolicitud,
        @RequestParam(required = false) String idSolSMO){
        return ResponseEntity.ok(altaComercioService.consultarEstado(canal, tipoDoc, nroTributario, idSolicitud, idSolSMO));
    }
    
    @GetMapping("/consultaTycs/idSolicitud/{idSolicitud}")
    public ResponseEntity<ConsultaTycsResponse> consultaTycs(@PathVariable String idSolicitud) {

    	ConsultaTycsResponse consultaTycsResponse = ConsultaTycsResponse
                .builder()
                .resultado(HttpStatus.OK.name())
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .idSolicitud(idSolicitud)
                .listaIdsTycs(this.tycsService.getTycsPorIdSolicitud(idSolicitud))
                .build();
        
        return ResponseEntity.ok(consultaTycsResponse);
    }
    
    @PostMapping("/cancelacion")
    public ResponseEntity<CancelacionResponse> cancelacion(@Valid @RequestBody CancelacionRequest cancelacionRequest) {
    	
        altaComercioService.cancelacion(cancelacionRequest, new AltaComercioScopeHelper(null));

        CancelacionResponse cancelacion = CancelacionResponse
                .builder()
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .build();
        
        return ResponseEntity.ok(cancelacion);
    }
    
}
