package ar.com.macro.adapter.controllers;

import ar.com.macro.adapter.services.DesbordePrestamoService;
import ar.com.macro.adapter.services.ProcesoService;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoActualizarRequest;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoIniciarRequest;
import ar.com.macro.core.application.incoming.response.DesbordePrestamoDetalleResponse;
import ar.com.macro.core.application.incoming.response.DesbordePrestamoResponse;
import javax.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequiredArgsConstructor
@RequestMapping("api/desbordePrestamo")
public class DesbordePrestamoController {

    private final DesbordePrestamoService desbordePrestamoService;
    private final ProcesoService procesoService;

    @PostMapping("/iniciar")
    public ResponseEntity<DesbordePrestamoResponse> iniciar(
            @Valid @RequestBody DesbordePrestamoIniciarRequest desbordePrestamoIniciarRequest) {

        String processInstanceId = desbordePrestamoService.iniciar(desbordePrestamoIniciarRequest);

        DesbordePrestamoResponse desbordePrestamoIniciarResponse = DesbordePrestamoResponse
                .builder()
                .resultado(HttpStatus.OK.name())
                .codResultado(HttpStatus.OK.value())
                .descripcion(HttpStatus.OK.series().name())
                .idSolicitud(processInstanceId)
                .estadoDelProceso(procesoService.getEstadoDelProceso(processInstanceId))
                .build();

        return ResponseEntity.ok().body(desbordePrestamoIniciarResponse);
    }

    @PostMapping("/actualizarEstadoLead")
    public ResponseEntity<HttpEntity> actualizarEstadoLead(
      @Valid @RequestBody DesbordePrestamoActualizarRequest desbordePrestamoActualizarRequest) {

        desbordePrestamoService.actualizarEstadoLead(desbordePrestamoActualizarRequest);

        return ResponseEntity.status(201).build();
    }
    
    @GetMapping("/consultaDetalle")
    @ResponseStatus(code = HttpStatus.OK)
    public DesbordePrestamoDetalleResponse consultaDetalle(@RequestParam String idCanal, @RequestParam String idSolicitud) {
    	return desbordePrestamoService.consultaDetalle(idSolicitud);
    }
    
}
