package ar.com.macro.adapter.services;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Service;

import ar.com.macro.bpm.scope.AltaAcuerdoCtaCteScopeHelper;
import ar.com.macro.bpm.scope.ScopeHelper;
import ar.com.macro.core.application.incoming.request.AltaAcuerdoCtaCteRequest;
import ar.com.macro.core.application.incoming.response.CondicionesResponse;

@Service
public class AltaAcuerdoCtaCteService extends CommonsService {

	private static final String VENTANA_HORARIA = "Lunes-viernes 8-20";

	public AltaAcuerdoCtaCteService(ProcesoService procesoService) {
		super(procesoService);
	}
	
	public String iniciar(AltaAcuerdoCtaCteRequest altaAcuerdoCtaCteRequest) {

		Map<String, Object> variables = new HashMap<>();

		variables.put(ScopeHelper.CANAL, altaAcuerdoCtaCteRequest.getCanal());
		variables.put(ScopeHelper.ID_COBIS, altaAcuerdoCtaCteRequest.getIdCobis());
		variables.put(AltaAcuerdoCtaCteScopeHelper.TIPO_PERSONA, altaAcuerdoCtaCteRequest.getTipoPersona());
		variables.put(AltaAcuerdoCtaCteScopeHelper.TIPO_DOCUMENTO, altaAcuerdoCtaCteRequest.getTipoDocumento());
		variables.put(AltaAcuerdoCtaCteScopeHelper.NRO_DOC, altaAcuerdoCtaCteRequest.getNroDoc());
		variables.put(AltaAcuerdoCtaCteScopeHelper.NRO_CUENTA, altaAcuerdoCtaCteRequest.getNroCuenta());
		variables.put(AltaAcuerdoCtaCteScopeHelper.OPERACION, altaAcuerdoCtaCteRequest.getOperacion());
		variables.put(AltaAcuerdoCtaCteScopeHelper.PLAZO, altaAcuerdoCtaCteRequest.getPlazo());
		variables.put(AltaAcuerdoCtaCteScopeHelper.RENOVACION, altaAcuerdoCtaCteRequest.isRenovacion());
		variables.put(AltaAcuerdoCtaCteScopeHelper.PERIODOS, altaAcuerdoCtaCteRequest.getPeriodos());
		variables.put(AltaAcuerdoCtaCteScopeHelper.MONTO, altaAcuerdoCtaCteRequest.getMonto());
		variables.put(AltaAcuerdoCtaCteScopeHelper.PORCEN_DISMINUC, altaAcuerdoCtaCteRequest.getPorcenDisminuc());
		variables.put(AltaAcuerdoCtaCteScopeHelper.LINEA_CREDITO, altaAcuerdoCtaCteRequest.getLineaCredito());
		variables.put(AltaAcuerdoCtaCteScopeHelper.TIPO_LIQUIDACION, altaAcuerdoCtaCteRequest.getTipoLiquidacion());
		variables.put(AltaAcuerdoCtaCteScopeHelper.DESVIO_TASA, altaAcuerdoCtaCteRequest.getDesvioTasa());
		variables.put(AltaAcuerdoCtaCteScopeHelper.TIPO_DESVIO, altaAcuerdoCtaCteRequest.getTipoDesvio());
		variables.put(AltaAcuerdoCtaCteScopeHelper.SOL_TIPO_DOC, altaAcuerdoCtaCteRequest.getSolTipoDoc());
		variables.put(AltaAcuerdoCtaCteScopeHelper.SOL_NUM_DOC, altaAcuerdoCtaCteRequest.getSolNumDoc());
		variables.put(AltaAcuerdoCtaCteScopeHelper.SOL_CARACTER_INVOCADO, altaAcuerdoCtaCteRequest.getSolCaracterInvocado());
		variables.put(AltaAcuerdoCtaCteScopeHelper.SOL_PARTICIPACION_SEG, altaAcuerdoCtaCteRequest.getSolParticipacionSeg());

		return procesoService.ejecutar(AltaAcuerdoCtaCteScopeHelper.PROCESS_KEY, variables);
	}

	public CondicionesResponse buscarParametros(String canal, String idProducto) {
		// TODO[EDPC-99-juliano] descomentar quando serviço estiver disponivel
		// TODO[EDPC-99-juliano] CondicionesResponse parametrosCondiciones = this.esbClient.getCondicionesESB(canal,idProducto);
		CondicionesResponse parametrosCondiciones = new CondicionesResponse();
		List<Integer> plazosRenovacion = new ArrayList<>();
		plazosRenovacion.add(30);
		parametrosCondiciones.setPlazosRenovacion(plazosRenovacion);
		parametrosCondiciones.setTopeCanal(BigDecimal.valueOf(100000));
		parametrosCondiciones.setTopePFseguro(BigDecimal.valueOf(80000));
		parametrosCondiciones.setTopePJGarantia(BigDecimal.valueOf(90000));
		parametrosCondiciones.setTasaFija30Dias(BigDecimal.valueOf(10.5));
		parametrosCondiciones.setTasasVariables(BigDecimal.valueOf(5.5));
		parametrosCondiciones.setTasaPerfilAcopiadorSoja(BigDecimal.valueOf(3.5));
		parametrosCondiciones.setTasaPerfilMiPyme(BigDecimal.valueOf(5.5));
		parametrosCondiciones.setVentanaHoraria(VENTANA_HORARIA);

		return parametrosCondiciones;
	}

	
}
