package ar.com.macro.adapter.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.stereotype.Service;

import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.bpm.scope.ScopeHelper;
import ar.com.macro.core.application.incoming.request.AltaComercioRequest;
import ar.com.macro.core.application.incoming.response.ConsultaEstadoSolicitudResponse;
import ar.com.macro.core.application.incoming.response.SmoAltaComercioResponse;
import ar.com.macro.infraestructure.exception.NotFoundException;


@Service
public class AltaComercioService extends CommonsService {

    public AltaComercioService(ProcesoService procesoService) {
		super(procesoService);
	}

	public String iniciar(AltaComercioRequest altaComercioRequest) {

        Map<String, Object> variables = new HashMap<>();

        variables.put(ScopeHelper.CANAL, altaComercioRequest.getCanal());
        variables.put(ScopeHelper.ID_COBIS, altaComercioRequest.getIdCobis());
        variables.put(AltaComercioScopeHelper.CUIT, altaComercioRequest.getCuit());
        variables.put(AltaComercioScopeHelper.SUCURSAL, altaComercioRequest.getSucursal());
        variables.put(AltaComercioScopeHelper.FORMA_DE_PAGO, altaComercioRequest.getFormaDePago());
        variables.put(AltaComercioScopeHelper.CUENTA_ACREDITACION, altaComercioRequest.getCuentaAcreditacion());
        variables.put(AltaComercioScopeHelper.TIPOIIBB, altaComercioRequest.getTipoiibb());
        variables.put(AltaComercioScopeHelper.NROIIBB, altaComercioRequest.getNroiibb());
        variables.put(AltaComercioScopeHelper.MAIL_CONTACTO, altaComercioRequest.getMailContacto());
        variables.put(AltaComercioScopeHelper.TIPO_TELEFONO, altaComercioRequest.getTipoTelefono());
        variables.put(AltaComercioScopeHelper.CODIGO_AREA, altaComercioRequest.getCodigoArea());
        variables.put(AltaComercioScopeHelper.TEL_CONTACTO, altaComercioRequest.getTelContacto());
        variables.put(AltaComercioScopeHelper.NOMBRE_FANTASIA, altaComercioRequest.getNombreFantasia());
        variables.put(AltaComercioScopeHelper.MODALIDAD_VENTA, altaComercioRequest.getModalidadVenta());
        variables.put(AltaComercioScopeHelper.TIPO_PERSONA, altaComercioRequest.getTipoPersona());
        variables.put(AltaComercioScopeHelper.PROVINCIA, altaComercioRequest.getProvincia());
        variables.put(AltaComercioScopeHelper.LOCALIDAD, altaComercioRequest.getLocalidad());
        variables.put(AltaComercioScopeHelper.CALLE, altaComercioRequest.getCalle());
        variables.put(AltaComercioScopeHelper.NRO_CALLE, altaComercioRequest.getNroCalle());
        variables.put(AltaComercioScopeHelper.PISO, altaComercioRequest.getPiso());
        variables.put(AltaComercioScopeHelper.DEPARTAMENTO, altaComercioRequest.getDepartamento());
        variables.put(AltaComercioScopeHelper.CP, altaComercioRequest.getCp());
        variables.put(AltaComercioScopeHelper.MARCA, altaComercioRequest.getMarca());
        variables.put(AltaComercioScopeHelper.PRODUCTO_DISPO, altaComercioRequest.getProductoDispo());
        variables.put(AltaComercioScopeHelper.ACTIVIDAD_AFIP, altaComercioRequest.getActividadAfip());
        variables.put(AltaComercioScopeHelper.CANT_TERMINALES, altaComercioRequest.getCantTerminales());
        variables.put(AltaComercioScopeHelper.TIPO_TERMINAL, altaComercioRequest.getTipoTerminal());
        variables.put(AltaComercioScopeHelper.CUOTAS, altaComercioRequest.getCuotas());
        variables.put(AltaComercioScopeHelper.PLAN, altaComercioRequest.getPlan());
        variables.put(AltaComercioScopeHelper.GATEWAY, altaComercioRequest.getGateway());
        variables.put(AltaComercioScopeHelper.COD_ADMINISTRADORA, altaComercioRequest.getCodAdministradora());
        variables.put(AltaComercioScopeHelper.RESP_NOMBRE, altaComercioRequest.getRespNombre());
        variables.put(AltaComercioScopeHelper.RESP_APELLIDO, altaComercioRequest.getRespApellido());
        variables.put(AltaComercioScopeHelper.RESP_NACIONALIDAD, altaComercioRequest.getRespNacionalidad());
        variables.put(AltaComercioScopeHelper.RESP_LUGAR_NAC, altaComercioRequest.getRespLugarNac());
        variables.put(AltaComercioScopeHelper.RESP_PAIS_RESI, altaComercioRequest.getRespPaisResi());
        variables.put(AltaComercioScopeHelper.RESP_PARTICIPACION, altaComercioRequest.getRespParticipacion());
        variables.put(AltaComercioScopeHelper.RESP_TIPO_DOC, altaComercioRequest.getRespTipoDoc());
        variables.put(AltaComercioScopeHelper.RESP_NUM_DOC, altaComercioRequest.getRespNumDoc());
        variables.put(AltaComercioScopeHelper.RESP_CUIT, altaComercioRequest.getRespCuit());
        variables.put(AltaComercioScopeHelper.RESP_PROFESION, altaComercioRequest.getRespProfesion());
        variables.put(AltaComercioScopeHelper.RESP_PEP, altaComercioRequest.getRespPep());
        variables.put(AltaComercioScopeHelper.RESP_TIPO_PEP, altaComercioRequest.getRespTipoPep());
        variables.put(AltaComercioScopeHelper.FECHA_CONTRATO_CONSTITUTIVO, altaComercioRequest.getFechaContratoConstitutivo());
        variables.put(AltaComercioScopeHelper.NUMERO_INSCRIPTION_REGISTRAL, altaComercioRequest.getNumeroInscriptionRegistral());
        variables.put(AltaComercioScopeHelper.FECHA_INSCRIPTION_REGISTAL, altaComercioRequest.getFechaInstrictionRegistral());
        
        return procesoService.ejecutar(AltaComercioScopeHelper.PROCESS_KEY, variables);
    }

    //TODO[EDPC-108-felipe] não temos ainda serviço no ESB
    public List<ConsultaEstadoSolicitudResponse> consultarEstado(String canal, String tipoDoc, Long nroTributario, String idSolicitud, String idSolSMO) {
        List<ConsultaEstadoSolicitudResponse> consultaEstadoSolicitudResponse = new ArrayList<>();
        Optional.of(this.procesoService.buscaProcessos(canal, tipoDoc, nroTributario, idSolicitud, idSolSMO)).ifPresent(instance ->
            instance.forEach(p -> {
                SmoAltaComercioResponse response = new SmoAltaComercioResponse().mockResponse(p.getId());
                if (response != null) {
                    consultaEstadoSolicitudResponse.add(ConsultaEstadoSolicitudResponse.builder()
                            .estado(response.getEncabezado().getEstado())
                            .numeroComercio(response.getEncabezado().getNumeroComercio())
                            .modoEntrada(response.getEncabezado().getModoEntrada())
                            .tipoSolicitud(response.getEncabezado().getTipoSolicitud())
                            .idSolicitud(response.getEncabezado().getIdSolicitud()).build());
                }
            })
        );
        
        if (consultaEstadoSolicitudResponse.isEmpty()) {
        	// TODO[] properties
            throw new NotFoundException("Ninguna solicitud encontrada");
        }

        return consultaEstadoSolicitudResponse;
    }

}
