package ar.com.macro.adapter.services;

import java.util.HashMap;
import java.util.Map;

import org.camunda.bpm.engine.runtime.MessageCorrelationBuilder;
import org.springframework.stereotype.Service;

import ar.com.macro.bpm.scope.ScopeHelper;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.core.application.incoming.request.FirmaRequest;
import ar.com.macro.infraestructure.exception.BusinessException;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
abstract class CommonsService {

    protected final ProcesoService procesoService;

    public String recibirFirma(FirmaRequest firmaRequest, ScopeHelper scopeHelper) {

        Map<String, Object> variables = new HashMap<>();

        variables.put(ScopeHelper.CANAL, firmaRequest.getCanal());
        variables.put(ScopeHelper.ID_SOLICITUD, firmaRequest.getIdSolicitud());
        variables.put(ScopeHelper.ID_COBIS, firmaRequest.getIdCobis());
        variables.put(ScopeHelper.FIRM_NOMBRE, firmaRequest.getFirmNombre());
        variables.put(ScopeHelper.FIRM_APELLI, firmaRequest.getFirmApelli());
        variables.put(ScopeHelper.FIRM_FECHA_NAC, firmaRequest.getFirmFechaNac());
        variables.put(ScopeHelper.FIRM_NACIONALIDAD, firmaRequest.getFirmNacionalidad());
        variables.put(ScopeHelper.FIRM_LUGAR_NAC, firmaRequest.getFirmLugarNac());
        variables.put(ScopeHelper.FIRM_SEXO, firmaRequest.getFirmSexo());
        variables.put(ScopeHelper.FIRM_EST_CIVIL, firmaRequest.getFirmEstCivil());
        variables.put(ScopeHelper.FIRM_TIPO_DOC, firmaRequest.getFirmTipoDoc());
        variables.put(ScopeHelper.FIRM_NUM_DOC, firmaRequest.getFirmNumDoc());
        variables.put(ScopeHelper.FIRM_CUIT, firmaRequest.getFirmCuit());
        variables.put(ScopeHelper.FIRM_PROV, firmaRequest.getFirmProv());
        variables.put(ScopeHelper.FIRM_LOCALIDAD, firmaRequest.getFirmLocalidad());
        variables.put(ScopeHelper.FIRM_CALLE, firmaRequest.getFirmCalle());
        variables.put(ScopeHelper.FIRM_NRO_CALLE, firmaRequest.getFirmNroCalle());
        variables.put(ScopeHelper.FIRM_CP, firmaRequest.getFirmCp());
        variables.put(ScopeHelper.FIRM_PEP, firmaRequest.getFirmPep());
        variables.put(ScopeHelper.FIRM_TIPO_PEP, firmaRequest.getFirmTipoPep());
        variables.put(ScopeHelper.LISTA_TYCS, firmaRequest.getIdTYC());

        MessageCorrelationBuilder correlation = this.procesoService.createMessageCorrelation(scopeHelper.getMessageKeyFirmaRealizada());
        correlation.processInstanceId(firmaRequest.getIdSolicitud());
        correlation.setVariables(variables);
        correlation.correlate();

        return firmaRequest.getIdSolicitud();
    }

    public void cancelacion(CancelacionRequest cancelacionRequest, ScopeHelper scopeHelper) {
    	
        Map<String, Object> variables = new HashMap<>();

        variables.put(ScopeHelper.CANAL, cancelacionRequest.getIdCanal());
        variables.put(ScopeHelper.MOTIVO_CANCELACION, cancelacionRequest.getMotivoCancelacion());
        variables.put(ScopeHelper.CUIT_FIRMANTE, cancelacionRequest.getCuitFirmante());
        
        MessageCorrelationBuilder correlation = this.procesoService.createMessageCorrelation(scopeHelper.getMessageKeyCancelacionSolicitada());
        correlation.processInstanceId(cancelacionRequest.getIdSolicitud());
        correlation.setVariables(variables);
        correlation.correlate();
        
        if (procesoService.isActivityInstance(cancelacionRequest.getIdSolicitud())) {
        	String resposta = String.valueOf(procesoService.getVariable(cancelacionRequest.getIdSolicitud(), ScopeHelper.MOTIVO_CANCELACION));
        	throw new BusinessException(resposta);
        }
    }

}
