package ar.com.macro.adapter.services;

import ar.com.macro.bpm.scope.DesbordePrestamoScopeHelper;
import ar.com.macro.bpm.scope.ScopeHelper;
import ar.com.macro.core.application.incoming.request.CuotaRequest;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoActualizarRequest;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoIniciarRequest;
import ar.com.macro.core.application.incoming.response.CuotaResponse;
import ar.com.macro.core.application.incoming.response.DesbordePrestamoDetalleResponse;
import ar.com.macro.infraestructure.exception.NotFoundException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import lombok.RequiredArgsConstructor;
import org.camunda.bpm.engine.runtime.MessageCorrelationBuilder;
import org.springframework.stereotype.Service;


@Service
@RequiredArgsConstructor
public class DesbordePrestamoService {

    private final ProcesoService procesoService;
    
    public String iniciar(DesbordePrestamoIniciarRequest desbordePrestamoIniciarRequest) {

        Map<String, Object> variables = new HashMap<>();

        variables.put(ScopeHelper.CANAL, desbordePrestamoIniciarRequest.getCanal());
        variables.put(DesbordePrestamoScopeHelper.CUIT, desbordePrestamoIniciarRequest.getCuit());
        variables.put(DesbordePrestamoScopeHelper.PASO, desbordePrestamoIniciarRequest.getPaso());
        variables.put(DesbordePrestamoScopeHelper.RAZON_SOCIAL, desbordePrestamoIniciarRequest.getRazonSocial());
        variables.put(DesbordePrestamoScopeHelper.PRODUCTO_ID, desbordePrestamoIniciarRequest.getProductoId());
        variables.put(DesbordePrestamoScopeHelper.PRODUCTO_DESC, desbordePrestamoIniciarRequest.getProductoDesc());
        variables.put(DesbordePrestamoScopeHelper.FECHA, desbordePrestamoIniciarRequest.getFecha());
        variables.put(DesbordePrestamoScopeHelper.VALOR_DISPONIBLE, desbordePrestamoIniciarRequest.getValorDisponible());
        variables.put(DesbordePrestamoScopeHelper.VALOR_CALIFICADO, desbordePrestamoIniciarRequest.getValorCalificado());
        variables.put(DesbordePrestamoScopeHelper.MONTO_SOLICITADO, desbordePrestamoIniciarRequest.getMontoSolicitado());
        variables.put(DesbordePrestamoScopeHelper.PER_PAGO, desbordePrestamoIniciarRequest.getPerPago());
        variables.put(DesbordePrestamoScopeHelper.PLAZO, desbordePrestamoIniciarRequest.getPlazo());
        variables.put(DesbordePrestamoScopeHelper.CUOTAS, desbordePrestamoIniciarRequest.getCuotas());
        variables.put(DesbordePrestamoScopeHelper.VALOR_CUOTAS, desbordePrestamoIniciarRequest.getValorCuotas());
        variables.put(DesbordePrestamoScopeHelper.TASA_FIJA, desbordePrestamoIniciarRequest.getTasaFija());
        variables.put(DesbordePrestamoScopeHelper.CFT, desbordePrestamoIniciarRequest.getCft());
        variables.put(DesbordePrestamoScopeHelper.GASTOS, desbordePrestamoIniciarRequest.getGastos());
        variables.put(DesbordePrestamoScopeHelper.SISTEMAS_AMORTIZACION, desbordePrestamoIniciarRequest.getSistemaAmortizacion());
        variables.put(DesbordePrestamoScopeHelper.CUENTA, desbordePrestamoIniciarRequest.getCuenta());
        variables.put(DesbordePrestamoScopeHelper.TIPO_CUENTA, desbordePrestamoIniciarRequest.getTipoCuenta());
        variables.put(DesbordePrestamoScopeHelper.CUENTA_DESENBOLSO , desbordePrestamoIniciarRequest.getCuentaDesenbolso());
        variables.put(DesbordePrestamoScopeHelper.MONEDA , desbordePrestamoIniciarRequest.getMoneda());
        variables.put(DesbordePrestamoScopeHelper.TIPO_DOCUMENTO , desbordePrestamoIniciarRequest.getTipoDocumento());
        variables.put(DesbordePrestamoScopeHelper.NUMERO_DOC , desbordePrestamoIniciarRequest.getNumeroDoc());
        variables.put(DesbordePrestamoScopeHelper.TIPO_PRESTAMO , desbordePrestamoIniciarRequest.getTipoPrestamo());
        variables.put(DesbordePrestamoScopeHelper.NUMERO_CLIENTE , desbordePrestamoIniciarRequest.getNumeroCliente());
        variables.put(DesbordePrestamoScopeHelper.LINEA_COMERCIAL , desbordePrestamoIniciarRequest.getLineaComercial());
        variables.put(DesbordePrestamoScopeHelper.SUB_LINEA_COMERCIAL , desbordePrestamoIniciarRequest.getSublineaComercial());
        variables.put(DesbordePrestamoScopeHelper.MERCADO , desbordePrestamoIniciarRequest.getMercado());
        variables.put(DesbordePrestamoScopeHelper.SUB_MERCADO , desbordePrestamoIniciarRequest.getSubmercado());
        variables.put(DesbordePrestamoScopeHelper.CUOTA , desbordePrestamoIniciarRequest.getCuota());
				variables.put(DesbordePrestamoScopeHelper.NOMBRE_SOLICITANTE, desbordePrestamoIniciarRequest.getNombreSolicitante());
				variables.put(DesbordePrestamoScopeHelper.APELLIDO_SOLICITANTE, desbordePrestamoIniciarRequest.getApellidoSolicitante());
				variables.put(DesbordePrestamoScopeHelper.TELEFONO_SOLICITANTE, desbordePrestamoIniciarRequest.getTelefonoSolicitante());
				variables.put(DesbordePrestamoScopeHelper.MAIL_SOLICITANTE, desbordePrestamoIniciarRequest.getMailSolicitante());

        return procesoService.ejecutar(DesbordePrestamoScopeHelper.PROCESS_KEY, variables);
    }

    public void actualizarEstadoLead(DesbordePrestamoActualizarRequest desbordePrestamoActualizarRequest){
        try {
					procesoService.actualizarDesbordePrestamoPorId(desbordePrestamoActualizarRequest);
					MessageCorrelationBuilder correlation = this.procesoService.createMessageCorrelation("DesbordeDePrestamoCambioEstadoRecebido");
					correlation.processInstanceId(desbordePrestamoActualizarRequest.getIdSolicitud());
					correlation.correlate();

        }catch (Exception e){
            throw new NotFoundException("Solicitud no encontrada");
        }
    }

    public DesbordePrestamoDetalleResponse consultaDetalle(String idSolicitud) {
    	if (!procesoService.isActivityInstance(idSolicitud)) {
    		throw new NotFoundException("Solicitud no puede ser encontrada");
    	}
    	
    	DesbordePrestamoDetalleResponse response = DesbordePrestamoDetalleResponse.builder()
				.idSolicitud(idSolicitud)
				.idCanal(String.valueOf(procesoService.getVariable(idSolicitud, ScopeHelper.CANAL)))
				.idCobis((Integer)procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.NUMERO_CLIENTE))
				.cuit((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUIT))
		    	.tipoDocumento(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TIPO_DOCUMENTO)))
		    	.numeroDocumento((Long) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.NUMERO_DOC))
		    	.idProducto((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PRODUCTO_ID))
		    	.descripcionProducto(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PRODUCTO_DESC)))
		    	.pasoAlcanzado(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PASO)))
		    	.fecha((LocalDateTime) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.FECHA))
		    	.valorDisponible((Double) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.VALOR_DISPONIBLE))
		    	.montoSolicitado((Double) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.MONTO_SOLICITADO))
		    	.periodoPago(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PER_PAGO)))
		    	.plazo((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PLAZO))
		    	.cuotas((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUOTAS))
		    	.valorCuotas((Double) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.VALOR_CUOTAS))
		    	.tasaFija(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TASA_FIJA)))
		    	.costoFinancieroTotal(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CFT)))
		    	.gastos((Double) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.GASTOS))
		    	.sistemaAmortizacion(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.SISTEMAS_AMORTIZACION)))
		    	.cuenta((Long) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUENTA))
		    	.tipoCuenta(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TIPO_CUENTA)))
		    	.cuentaDesenbolso((Long) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUENTA_DESENBOLSO))
		    	.moneda(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.MONEDA)))
		    	.tipoPrestamo(String.valueOf(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TIPO_PRESTAMO)))
		    	.lineaComercial((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.LINEA_COMERCIAL))
		    	.sublineaComercial((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.SUB_LINEA_COMERCIAL))
		    	.mercado((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.MERCADO))
		    	.submercado((Integer) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.SUB_MERCADO))
		    	.valorCalificado((Double) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.VALOR_CALIFICADO))
		    	.build();

				ArrayList<CuotaRequest> cuotaItems = (ArrayList<CuotaRequest>) procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUOTA);
				
				if (cuotaItems != null && !cuotaItems.isEmpty()) {
					ArrayList<CuotaResponse> cuotaItemsResponse = new ArrayList<>();
					
					cuotaItems.stream().forEach(cuota ->
						cuotaItemsResponse.add(
							CuotaResponse.builder()
								.numeroCuota(cuota.getNumero())
								.vencimiento(cuota.getVencimiento())
								.capitalAdeudo(cuota.getCapitalAdeudado())
								.capital(cuota.getCapital())
								.intereses(cuota.getIntereses())
								.seguro(cuota.getSeguro())
								.rubros(cuota.getRubros())
								.montoTotal(cuota.getTotalPagar())
								.iva(cuota.getIva())
								.build())
					);
					
					response.setCuota(cuotaItemsResponse);
				}
				
    	return response;
    }
    
}
