package ar.com.macro.adapter.services;

import java.util.List;
import java.util.Map;

import javax.validation.constraints.NotBlank;

import org.camunda.bpm.engine.ProcessEngine;
import org.camunda.bpm.engine.RuntimeService;
import org.camunda.bpm.engine.history.HistoricProcessInstance;
import org.camunda.bpm.engine.history.HistoricVariableInstance;
import org.camunda.bpm.engine.runtime.MessageCorrelationBuilder;
import org.camunda.bpm.engine.runtime.ProcessInstance;
import org.camunda.bpm.engine.variable.Variables;
import org.springframework.stereotype.Service;

import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.bpm.scope.DesbordePrestamoScopeHelper;
import ar.com.macro.bpm.scope.ScopeHelper;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoActualizarRequest;
import ar.com.macro.infraestructure.exception.BusinessException;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class ProcesoService {

    private final RuntimeService runtimeService;

    private final ProcessEngine processEngine;

    public MessageCorrelationBuilder createMessageCorrelation(@NotBlank String nombre) {
        return runtimeService.createMessageCorrelation(nombre);
    }
    
    public boolean isActivityInstance(String idSolicitud) {
    	return runtimeService.getActivityInstance(idSolicitud) != null;
    }
    
    public Object getVariable(String idSolicitud, String variable) {
    	return runtimeService.getVariable(idSolicitud, variable);
    }

    public String getEstadoDelProceso(String idSolicitud){
        return String.valueOf(runtimeService.getVariable(idSolicitud, ScopeHelper.ESTADO_DEL_PROCESO));
    }
 
    public String ejecutar(@NotBlank String nombre, Map<String, Object> variables) {
        if (this.procesoPermitidoEjecutar(nombre)) {
            return runtimeService.startProcessInstanceByKey(nombre, variables).getProcessInstanceId();
        } else {
            // TODO[] properties
            String error = String.format("El proceso %s no se encuentra en horario habilitado para su ejecución", nombre);
            throw new BusinessException(error);
        }
    }

    public List<HistoricProcessInstance> buscaProcessos(String canal, String tipoDoc, Long nroTributario, String idSolicitud, String idSolSMO) {
        if (idSolicitud == null) {
            return processEngine.getHistoryService()
                    .createHistoricProcessInstanceQuery()
                    .processDefinitionKey(AltaComercioScopeHelper.PROCESS_KEY)
                    .variableValueEquals(AltaComercioScopeHelper.RESP_NUM_DOC, nroTributario)
                    .variableValueEquals(ScopeHelper.CANAL, canal)
                    .variableValueEquals(AltaComercioScopeHelper.RESP_TIPO_DOC, tipoDoc)
                    .unlimitedList();
        } else {
            return processEngine.getHistoryService()
                    .createHistoricProcessInstanceQuery()
                    .processDefinitionKey(AltaComercioScopeHelper.PROCESS_KEY)
                    .variableValueEquals(AltaComercioScopeHelper.RESP_NUM_DOC, nroTributario)
                    .variableValueEquals(ScopeHelper.CANAL, canal)
                    .variableValueEquals(AltaComercioScopeHelper.RESP_TIPO_DOC, tipoDoc)
                    .processInstanceId(idSolicitud)
                    .unlimitedList();
        }
    }

    private boolean procesoPermitidoEjecutar(@NotBlank String nombre) {
        ProcessInstance pi = this.runtimeService.startProcessInstanceByKey("HorariosDisponibilidadProcesos",
                Variables.createVariables().putValue("procesoKey", nombre));
        
        List<HistoricVariableInstance> variables = this.processEngine.getHistoryService()
                .createHistoricVariableInstanceQuery().processInstanceId(pi.getId()).list();
        
        return variables.stream().filter(v -> v.getName().equalsIgnoreCase("habilitado")).findFirst()
                .map(h -> Boolean.parseBoolean(h.getValue().toString())).orElse(false);
    }

    public void actualizarDesbordePrestamoPorId(DesbordePrestamoActualizarRequest desbordePrestamoActualizarRequest) {
        runtimeService.setVariable(desbordePrestamoActualizarRequest.getIdSolicitud(), DesbordePrestamoScopeHelper.ID_USUARIO, desbordePrestamoActualizarRequest.getIdUsuario());
        runtimeService.setVariable(desbordePrestamoActualizarRequest.getIdSolicitud(), ScopeHelper.CANAL, desbordePrestamoActualizarRequest.getIdCanal());
        runtimeService.setVariable(desbordePrestamoActualizarRequest.getIdSolicitud(), DesbordePrestamoScopeHelper.DESC_USUARIO, desbordePrestamoActualizarRequest.getDescUsuario());
        runtimeService.setVariable(desbordePrestamoActualizarRequest.getIdSolicitud(), DesbordePrestamoScopeHelper.ESTADO_LEAD, desbordePrestamoActualizarRequest.getEstadoLead());
        if(desbordePrestamoActualizarRequest.getObservaciones() != null){
            runtimeService.setVariable(desbordePrestamoActualizarRequest.getIdSolicitud(), DesbordePrestamoScopeHelper.OBSERVACIONES, desbordePrestamoActualizarRequest.getObservaciones());
        }
    }
}
