package ar.com.macro.adapter.services;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.Bucket;
import com.amazonaws.services.s3.model.ListObjectsV2Result;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotBlank;
import java.io.File;
import java.util.List;
import java.util.Optional;

@Slf4j
@Service
@RequiredArgsConstructor
public class S3Service {

    private final AmazonS3 amazonS3Client;
    @Setter
    @Value("${aws.bucket.name}")
    private String bucketName;

    public List<Bucket> listarBuckets() {
        List<Bucket> buckets = amazonS3Client.listBuckets();
        buckets.forEach(bucket -> log.info("Bucket {} created at {}", bucket.getName(), bucket.getCreationDate()));
        return buckets;
    }

    public List<S3ObjectSummary> listarArchivosPorBucket(String bucketNombre) {
        String bucket = Optional.ofNullable(bucketNombre).orElse(bucketName);
        ListObjectsV2Result result = amazonS3Client.listObjectsV2(bucket);
        List<S3ObjectSummary> objectSummaries = result.getObjectSummaries();
        objectSummaries.forEach(os -> log.info("* {}", os.getKey()));
        return objectSummaries;
    }

    public void subirArchivo(@NotBlank String fileKey, @NotBlank File file) {
        log.info("Subiendo el archivo {} para el bucket {}", fileKey, bucketName);
        amazonS3Client.putObject(bucketName, fileKey, file);
    }

    public void suprimirArchivo(@NotBlank String fileKey) {
        log.info("Suprimiendo el archivo {} para del bucket {}", fileKey, bucketName);
        amazonS3Client.deleteObject(bucketName, fileKey);
    }

}
