package ar.com.macro.adapter.services;

import ar.com.macro.adapter.aws.secrets.AwsEsbSecret;
import ar.com.macro.core.application.entities.Token;
import ar.com.macro.core.application.outcoming.TokenEsbClient;
import ar.com.macro.core.application.repository.TokenRepository;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class TokenInterceptorService implements RequestInterceptor {

    private final TokenEsbClient tokenEsbClient;
    private final TokenRepository tokenRepository;
    private final AwsEsbSecret awsEsbSecret;

    public String getToken() {

        List<Token> actualList = new ArrayList<>();

        Objects.requireNonNull(tokenRepository.findAll()).iterator().forEachRemaining(actualList::add);

        if (!actualList.isEmpty() && !isExpired(actualList.get(0))) {
            return actualList.get(0).getValue();
        }
        return renewToken().getValue();
    }

    private Map<String, String> generateHeaders() {
        Map<String, String> headers = new HashMap<>();
        headers.put("Accept", "*/*");
        headers.put("AcceptEncoding", "gzip,deflate, br");
        String encoding = Base64.getEncoder().encodeToString((this.awsEsbSecret.getUsername() + ":" +
                this.awsEsbSecret.getPassword()).getBytes());
        headers.put(HttpHeaders.AUTHORIZATION, "Basic " + encoding);
        return headers;
    }

    public Token renewToken() {
        String auth = String.valueOf(tokenEsbClient.getToken(this.generateHeaders()).headers().get("X-Authorization-token"));
        Token newToken = new Token();
        newToken.setValue(auth);
        newToken.setDate(new Date());
        tokenRepository.save(newToken);
        return newToken;
    }

    public boolean isExpired(Token token) {
        Instant inicio = token.getDate().toInstant();
        Instant fim = inicio.plus(25, ChronoUnit.MINUTES);
        return Instant.now().isAfter(fim);
    }

    @Override
    public void apply(RequestTemplate requestTemplate) {
        requestTemplate.header("Content-Type", "*/*");
        requestTemplate.header("Host", "camunda");
        requestTemplate.header("Accept", "application/json");
        requestTemplate.header("X-Authorization-impersonationLevel", "service");
        requestTemplate.header("X-Application-identity", this.awsEsbSecret.getApplicationId());
        requestTemplate.header("X-Authorization-token", getToken());
    }
}
