package ar.com.macro.core.application.delegate;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Optional;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;
import org.springframework.stereotype.Component;

import ar.com.macro.bpm.scope.AltaAcuerdoCtaCteScopeHelper;
import ar.com.macro.core.application.outcoming.EsbClient;
import ar.com.macro.core.application.outcoming.response.AperturaLineaResponse;
import ar.com.macro.core.application.outcoming.response.ElementsResponse;
import ar.com.macro.core.application.outcoming.response.RiesgosResponse;
import lombok.RequiredArgsConstructor;

@Component("ACCVerificarDisponible")
@RequiredArgsConstructor
public class ACCVerificarDisponibleDelegate implements JavaDelegate {

	public static final String ACUERDOCTA = "ACUERDOCTA";
	
    private final EsbClient esbClient;
    
    @Override
    public void execute(DelegateExecution delegateExecution) {
    	
    	final AltaAcuerdoCtaCteScopeHelper altaAcuerdoCtaCteScopeHelper = new AltaAcuerdoCtaCteScopeHelper(delegateExecution);

    	altaAcuerdoCtaCteScopeHelper.getOperacion();
    	
    	// TODO[EDPC-87-mario]: esses campos ainda não existe no processo de acuerdoCTACTE, serão fixos?
        String moneda = Optional.ofNullable(delegateExecution.getVariable("moneda")).orElse("80").toString();
        String linea = Optional.ofNullable(delegateExecution.getVariable("linea")).orElse("1").toString();
        String ente = String.valueOf(altaAcuerdoCtaCteScopeHelper.getIdCobis());

        RiesgosResponse riestosCalificacionEndeudamientoCliente = this.esbClient
        		.getRiestosCalificacionEndeudamientoCliente(ente, moneda, linea);
        
        ElementsResponse elementsResponse = new ElementsResponse();
        AperturaLineaResponse aperturaLineaResponse = new AperturaLineaResponse();
        
        aperturaLineaResponse.setOcLinea(ACUERDOCTA);
        aperturaLineaResponse.setOiDisponible(BigDecimal.ZERO);

        elementsResponse.setAperturaLineas(Collections.singletonList(aperturaLineaResponse));
        
        BigDecimal total = Optional.ofNullable(riestosCalificacionEndeudamientoCliente.getElements())
                .orElse(Collections.emptyList())
                .stream().findFirst().orElse(elementsResponse)
                .getAperturaLineas().stream()
                .filter(a -> a.getOdLinea().equalsIgnoreCase(ACUERDOCTA))
                .map(AperturaLineaResponse::getOiDisponible)
                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);

        altaAcuerdoCtaCteScopeHelper.setTotalDisponible(total);
    }
    
}
