package ar.com.macro.core.application.delegate;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;
import org.springframework.stereotype.Component;

import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.core.application.entities.enums.ModalidadVenta;
import ar.com.macro.core.application.outcoming.request.ComercioRequest;
import ar.com.macro.core.application.outcoming.request.DatosBancoRequest;
import ar.com.macro.core.application.outcoming.request.DatosComercioRequest;
import ar.com.macro.core.application.outcoming.request.DatosFirmanteRequest;
import ar.com.macro.core.application.outcoming.request.DatosPersonalesRequest;
import ar.com.macro.core.application.outcoming.request.DireccionRequest;
import ar.com.macro.core.application.outcoming.request.EsquemasRequest;
import ar.com.macro.core.application.outcoming.request.IdentificationRequest;
import ar.com.macro.core.application.outcoming.request.InformacionContactoRequest;
import ar.com.macro.core.application.outcoming.request.IngresosBrutosRequest;
import ar.com.macro.core.application.outcoming.request.MetodoPagoRequest;
import ar.com.macro.core.application.outcoming.request.ModalidadTarjetaFisicaRequest;
import ar.com.macro.core.application.outcoming.request.PersonaNaturalRequest;
import ar.com.macro.core.application.outcoming.request.PoliticoExpuestoRequest;
import ar.com.macro.core.application.outcoming.request.ReprogramacionTerminalRequest;
import ar.com.macro.core.application.outcoming.request.TelefonoRequest;
import ar.com.macro.core.application.outcoming.request.TelefonoAlternativoRequest;
import ar.com.macro.core.application.outcoming.request.TerminalesRequest;
import lombok.RequiredArgsConstructor;

@Component("AComAltaSMO")
@RequiredArgsConstructor
public class AComAltaSMODelegate implements JavaDelegate {

	private AltaComercioScopeHelper scopeHelper;
	
    @Override
    public void execute(DelegateExecution delegateExecution) {

        scopeHelper = new AltaComercioScopeHelper(delegateExecution);

        ComercioRequest comercioRequest = ComercioRequest.builder()
                .datosPersonales(buildDatosPersonales())
                .datosBanco(buildDatosBanco())
                .datosComercio(buildDatosComercio())
                .ingresosBrutos(buildIngresosBrutos())
                .condicionIva(scopeHelper.getCondicionIva())
                .sujetoObligado(scopeHelper.getSujetoObligado())
                .modoOperacion(scopeHelper.getModalidadVenta())
                .categoria(scopeHelper.getRespTipoPEP())
                .informacionContacto(buildInformacionContacto())
                .tipoOrden(1)
                .terminales(buildTerminales())
                .modalidadTarjetaFisica(buildModalidadTarjetaFisica())
                .reprogramacionTerminal(buildReprogramacionTerminal())
                .metodoPago(buildMetodoPago())
                .gateway(null) //TODO[EDPC-117-avalos] verificar valor
                .requestGateway(scopeHelper.getGateway())
                .requestPaymentButtom(null) //TODO[EDPC-117-avalos] verificar valor
                .tipoEstablecimiento(ModalidadVenta.ECOMMERCE.name().equalsIgnoreCase(scopeHelper.getModalidadVenta()) ? "T" : null)
                .cuitMayorista(scopeHelper.getCuit())
                .build();
        
        comercioRequest.toString();

        //TODO[EDPC-109-juliano] não temos ainda serviço no ESB, remove mock when service disponible
        //this.esbClient.altaComercio(comercioRequest);
        //TODO[EDPC-109-juliano] não temos ainda serviço no ESB, o que iremos gravar no retorno?
    }

	private DatosPersonalesRequest buildDatosPersonales() {
		return DatosPersonalesRequest.builder()
                .personaNatural(buildPersonaNatural())
                .build();
	}
	
	private PersonaNaturalRequest buildPersonaNatural() {
		return PersonaNaturalRequest.builder()
				.identification(scopeHelper.getCuit())
	            .nombre(scopeHelper.getFirmNombre())
	            .apelido(scopeHelper.getFirmApelli())
	            .genero(scopeHelper.getFirmSexo())
	            .email(scopeHelper.getFirmEmail())
	            .estadoCivil(scopeHelper.getFirmEstCivil())
	            .ocupacion(scopeHelper.getRespProfesion())
	            .direccion(buildDireccionPersonaNatural())
	            .datosFirmante(buildDatosFirmante())
	            .build();
	}

	private DireccionRequest buildDireccionPersonaNatural() {
		return DireccionRequest.builder()
                .calle(scopeHelper.getFirmCalle())
                .numero(scopeHelper.getFirmNroCalle())
                .piso(scopeHelper.getFirmPiso())
                .depto(scopeHelper.getFirmDepto())
                .cP(scopeHelper.getFirmCP())
                .provincia(scopeHelper.getFirmProv())
                .localidad(scopeHelper.getFirmLocalidad())
                .telefono(buildTelefone(scopeHelper.getFirmCodigoArea(), scopeHelper.getFirmTelefone()))
                .build();
	}
	
	private TelefonoRequest buildTelefone(Long codigoArea, Long telefono) {
		return TelefonoRequest.builder()
				.codigoArea(codigoArea)
				.numero(telefono)
				.build();
	}
	
	private TelefonoAlternativoRequest buildTelefoneAlternativo(Long codigoArea, Long telefono) {
		return TelefonoAlternativoRequest.builder()
				.codigoArea(codigoArea)
				.numero(telefono)
				.build();
	}

	private DatosFirmanteRequest buildDatosFirmante() {
		return DatosFirmanteRequest.builder()
				.identification(buildIdentification())
				.nombre(scopeHelper.getRespNombre())
	            .apelido(scopeHelper.getRespApellido())
	            .genero(scopeHelper.getFirmSexo())
	            .ocupacion(scopeHelper.getRespProfesion())
	            .politicoExpuesto(buildPoliticoExpuesto())
	            .direccionReal(
	                    scopeHelper.getCalle() + " " +
	                    scopeHelper.getNroCalle() + " " +
	                    scopeHelper.getPiso() + " " +
	                    scopeHelper.getDepartamento() + " " +
	                    scopeHelper.getLocalidad() + " " +
	                    scopeHelper.getProvincia()
	            )
	            .cP(scopeHelper.getCP())
	            .email(scopeHelper.getMailContacto())
	            .telefono(buildTelefone(scopeHelper.getCodigoArea(), scopeHelper.getTelContacto()))
	            .lugarNacimiento(scopeHelper.getFirmLugarNac())
	            .fechaNacimiento(scopeHelper.getFirmFechaNac())
	            .build();
	}
	
	private IdentificationRequest buildIdentification() {
        return IdentificationRequest.builder()
                .tipoIdentification(scopeHelper.getRespTipoDoc())
                .numeroIdentification(scopeHelper.getRespNumDoc())
                .build();
	}
	
	private PoliticoExpuestoRequest buildPoliticoExpuesto() {
		return PoliticoExpuestoRequest.builder()
				.estado(scopeHelper.getFirmPep())
				.categoriaId(scopeHelper.getFirmTipoPep())
				.build();
	}
	
	private DatosBancoRequest buildDatosBanco() {
		return DatosBancoRequest.builder()
				.cbu(scopeHelper.getNumeroCbu())
				.cuentaTipo(scopeHelper.getTipoCuenta())
				.cuentaNumero(scopeHelper.getCuentaAcreditacion())
				.sucursalNumero(scopeHelper.getOficinaCuenta())
				//TODO[EDPC-117-avalos] valor deve vir do processo e não harcode
				.codigoBanco(67)
				.build();
	}

	private DatosComercioRequest buildDatosComercio() {
		return DatosComercioRequest.builder()
				.nombreFantasia(scopeHelper.getNombreFantasia())
				.email(scopeHelper.getMailContacto())
				.direccion(buildDireccionComercio())
				.telefono(buildTelefone(scopeHelper.getCodigoArea(), scopeHelper.getTelContacto()))
				.build();
	}
	
	private DireccionRequest buildDireccionComercio() {
		return DireccionRequest.builder()
	            .calle(scopeHelper.getCalle())
	            .numero(scopeHelper.getNroCalle())
	            .piso(scopeHelper.getPiso())
	            .depto(scopeHelper.getDepartamento())
	            .cP(scopeHelper.getFirmCP())
	            .provincia(scopeHelper.getProvincia())
	            .localidad(scopeHelper.getLocalidad())
	            .build();
	}

	private IngresosBrutosRequest buildIngresosBrutos() {
		return IngresosBrutosRequest.builder()
				.condicion(scopeHelper.getTipoiibb())
				.numeroInscripto(scopeHelper.getNroiibb())
				.build();
	}
	
	private InformacionContactoRequest buildInformacionContacto() {
        return InformacionContactoRequest.builder()
                .nombre(scopeHelper.getRespNombre())
                .apellido(scopeHelper.getRespApellido())
                .email(scopeHelper.getMailContacto())
                .telefono(buildTelefone(scopeHelper.getCodigoArea(), scopeHelper.getTelContacto()))
                .telefonoAlternativo(buildTelefoneAlternativo(scopeHelper.getCodigoArea(), scopeHelper.getTelContacto()))
                .build();
	}
	
	private TerminalesRequest buildTerminales() {
        return TerminalesRequest.builder()
        		.tipoTerminal(scopeHelper.getTipoTerminal())
                .cantidad(scopeHelper.getCantTerminales())
                .tipoPlan(scopeHelper.getPlan())
                .build();
	}
	
	private ModalidadTarjetaFisicaRequest buildModalidadTarjetaFisica() {
	    return ModalidadTarjetaFisicaRequest.builder()
	            .esquemas(buildEsquemas())
	            .build();
	}
	
	private EsquemasRequest buildEsquemas() {
		return EsquemasRequest.builder()
				.esquema(null)
				.numeroEstablecimiento(null)
				.build();
	}
	
	private ReprogramacionTerminalRequest buildReprogramacionTerminal() {
    	return ReprogramacionTerminalRequest.builder()
    			.numeroTerminal(null)
    			.esquemas(buildEsquemas())
    			.build();
	}
	
	private MetodoPagoRequest buildMetodoPago() {
    	return MetodoPagoRequest.builder()
    			.metodoTerminal(2)
    			.numeroTarjeta(null)
    			.build();
	}

}
