package ar.com.macro.core.application.delegate;

import java.util.Objects;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;
import org.springframework.stereotype.Component;

import ar.com.macro.adapter.services.ClientesConsultaCuentasService;
import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.core.application.outcoming.response.ClientesConsultaCuentasResponse;
import ar.com.macro.core.application.outcoming.response.ElementsClientesConsultaCuentasResponse;
import ar.com.macro.infraestructure.exception.IntegrationException;
import lombok.RequiredArgsConstructor;

@Component("AComClientesConsultaCuentas")
@RequiredArgsConstructor
public class AComClientesConsultaCuentasDelegate implements JavaDelegate {

	private final ClientesConsultaCuentasService clientesConsultaCuentasService;

	@Override
	public void execute(DelegateExecution delegateExecution) {

		final AltaComercioScopeHelper scopeHelper = new AltaComercioScopeHelper(delegateExecution);

		ElementsClientesConsultaCuentasResponse elementsClientesConsultaCuentasResponse = this.clientesConsultaCuentasService
				.getClientesConsultaDeCuentas(scopeHelper.getIdCobis());

        ClientesConsultaCuentasResponse clientesConsultaCuentasResponse = 
        		elementsClientesConsultaCuentasResponse
        		.getElements()
        		.stream()
                .filter(clientesConsultaCuentas -> 
                	Objects.equals(scopeHelper.getCuentaAcreditacion(), clientesConsultaCuentas.getOnCuenta()))
                .findAny()
                .orElse(null);
		
        if (clientesConsultaCuentasResponse == null) {
        	//TODO[] properties
        	throw new IntegrationException("Ocurrió un error al intentar a consultar cuentas,"
        			+ "verificar si CuentaAcreditacion existe para IdCobis");
        }
        
		scopeHelper.setTipoCuenta(clientesConsultaCuentasResponse.getOcTipoCuenta());
		scopeHelper.setOficinaCuenta(clientesConsultaCuentasResponse.getOcOficina());
		scopeHelper.setProductoCuenta(clientesConsultaCuentasResponse.getOcProducto());
		scopeHelper.setCuentaAcreditacion(clientesConsultaCuentasResponse.getOnCuenta());
		scopeHelper.setMonedaCuenta(clientesConsultaCuentasResponse.getOcMoneda());
	}

}
