package ar.com.macro.core.application.log.filter;

import ar.com.macro.core.application.delegate.dto.ResponseDto;
import ar.com.macro.core.application.repository.LogginRepository;
import ar.com.macro.infraestructure.exception.IntegrationException;
import camundajar.impl.com.google.gson.Gson;
import lombok.AllArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;
import org.springframework.web.util.ContentCachingRequestWrapper;
import org.springframework.web.util.ContentCachingResponseWrapper;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component
@AllArgsConstructor
public class LoggingFilter extends OncePerRequestFilter {

    private static final Logger LOGGER = LoggerFactory.getLogger(LoggingFilter.class);
    private static final Set<String> ALLOWED_PATHS = Set.of("/api");
    private final LogginRepository logginRepository;

    @Override
    protected boolean shouldNotFilter(HttpServletRequest request) {
        String path = request.getServletPath();
        return ALLOWED_PATHS.stream().noneMatch(path::startsWith);
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {

    	ContentCachingRequestWrapper requestWrapper = new ContentCachingRequestWrapper(request);
        ContentCachingResponseWrapper responseWrapper = new ContentCachingResponseWrapper(response);

        long startTime = System.currentTimeMillis();

        filterChain.doFilter(requestWrapper, responseWrapper);

        long timeTaken = System.currentTimeMillis() - startTime;

        String requestBody = getStringValue(requestWrapper.getContentAsByteArray(), request.getCharacterEncoding());
        String responseBody = getStringValue(responseWrapper.getContentAsByteArray(), response.getCharacterEncoding());

        LOGGER.info(
                "FINISHED PROCESSING : METHOD={}; REQUESTURI={}; REQUEST PAYLOAD={}; RESPONSE CODE={}; RESPONSE={}; TIEMPO DE EJECUCION={}",
                request.getMethod(), request.getRequestURI(), requestBody, response.getStatus(), responseBody, timeTaken);
        responseWrapper.copyBodyToResponse();

        String idSolicitud = "";
        Long idCobis = null;
        
        //TODO[] ....
        if (requestBody.contains("idCobis")) {
        	idCobis = new Gson().fromJson(requestBody, ResponseDto.class).getIdCobis();
        }
        if (requestBody.contains("idSolicitud")) {
        	idSolicitud = new Gson().fromJson(requestBody, ResponseDto.class).getIdSolicitud();
        } else if (responseBody.contains("idSolicitud")) {
        	idSolicitud = new Gson().fromJson(responseBody, ResponseDto.class).getIdSolicitud();
        }

        Map<String, List<String>> headersMap = Collections.list(request.getHeaderNames())
                .stream()
                .collect(Collectors.toMap(
                        Function.identity(),
                        h -> Collections.list(request.getHeaders(h))
                ));
        
        String headers = headersMap.keySet().stream()
                .map(key -> key + "=" + headersMap.get(key))
                .collect(Collectors.joining(", ", "{", "}"));
        
        Logging log = Logging.builder()
                .fecha(LocalDateTime.now())
                .idProceso(idSolicitud) //TODO[maks] está correto esse parâmetro? 
                .idCliente(idCobis)
                .idSolicitud(idSolicitud)
                .servicio(request.getRequestURI())
                .request(requestBody.isBlank() ? null : requestBody)
                .response(responseBody.isBlank() ? null : responseBody)
                .tiempoEjecucion(timeTaken)
                .httpStatus(String.valueOf(response.getStatus()))
                .descripcionError(HttpStatus.valueOf(response.getStatus()).toString().equals("200 OK") ? null : HttpStatus.valueOf(response.getStatus()).toString().substring(4))
                .campoAuxiliar1("controller-transaction")
                .campoAuxiliar2("Headers " + headers)
                .build();

        this.logginRepository.save(log);
    }

    private String getStringValue(byte[] contentAsByteArray, String characterEncoding) {
        try {
            return new String(contentAsByteArray, 0, contentAsByteArray.length, characterEncoding);
        } catch (UnsupportedEncodingException e) {
        	//TODO[] properties
            throw new IntegrationException("Error al intentar recuperar el request o response body " + e);
        }
    }

}
