package ar.com.macro.core.application.outcoming;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;

import ar.com.macro.adapter.services.TokenInterceptorService;
import ar.com.macro.core.application.incoming.response.CondicionesResponse;
import ar.com.macro.core.application.outcoming.response.ElementsClientesConsultaCuentasResponse;
import ar.com.macro.core.application.outcoming.response.ElementsClientesConsultaGeneralResponse;
import ar.com.macro.core.application.outcoming.response.ElementsConsultaDetalleResponse;
import ar.com.macro.core.application.outcoming.response.RiesgosResponse;
import ar.com.macro.infraestructure.errorhandling.ErrorCode;
import lombok.extern.slf4j.Slf4j;

@FeignClient(name = "${esb.name}", url = "${esb.url}", 
		fallback = EsbClient.Fallback.class,
        configuration = {TokenInterceptorService.class})
public interface EsbClient {

    @GetMapping(value = "/services/Riesgos_CalificacionEndeudamientoCliente/V1/R0/?"
    		+ "i_ente={ente}"
    		+ "&i_c_moneda={moneda}"
            + "&i_c_linea={linea}", 
            consumes = MediaType.APPLICATION_JSON_VALUE)
    RiesgosResponse getRiestosCalificacionEndeudamientoCliente(@PathVariable("ente") String ente,
                                                               @PathVariable("moneda") String moneda, 
                                                               @PathVariable("linea") String linea);

    @GetMapping(value = "/services/Clientes_ConsultaCuentas/V1/R0/V1/R0?i_ente={iEnte}", 
    		consumes = MediaType.APPLICATION_JSON_VALUE)
    ElementsClientesConsultaCuentasResponse getClientesConsultaDeCuentas(@PathVariable("iEnte") Long iEnte);

    @GetMapping(value = "/services/Condiciones/TBD/V1/R0/?"
    		+ "canal={canal}"
    		+ "&id_producto={idProducto}", 
    		consumes = MediaType.APPLICATION_JSON_VALUE)
    CondicionesResponse getCondicionesEsb(@PathVariable("canal") String canal, 
    										@PathVariable("idProducto") String idProducto);

    @GetMapping(value = "/services/Pasivas_ConsultaDetalleCuenta/V1/R0?"
    		+ "TipoCuenta={tipoCuenta}"
    		+ "&NumeroCuenta={numeroCuenta}"
    		+ "&CodigoMoneda={codigoMoneda}", 
    		consumes = MediaType.APPLICATION_JSON_VALUE)
    ElementsConsultaDetalleResponse getClientesConsultaDetalleCuentas(@RequestParam("TipoCuenta") String tipoCuenta,
                                                                      @RequestParam("NumeroCuenta") Long numeroCuenta,
                                                                      @RequestParam("CodigoMoneda") String codigoMoneda);

	@GetMapping(value = "/services/Clientes_ConsultaGeneral/V1/R0/?i_Ccliente={icCliente}", 
			consumes = MediaType.APPLICATION_JSON_VALUE)
	ElementsClientesConsultaGeneralResponse getClientesConsultaGeneral(@PathVariable("icCliente") Long icCliente);

	
	@GetMapping(value = "/services/Clientes_ConsultaGeneral/V1/R0/?i_Cdocumento={tipoDocumento}&i_Ndocumento={numeroIdentificacion}", 
			consumes = MediaType.APPLICATION_JSON_VALUE)
	ElementsClientesConsultaGeneralResponse getClientesConsultaGeneralPorCuit(@PathVariable("numeroIdentificacion") String icNumeroIdentificacion, 
																	  @PathVariable("tipoDocumento") String icTipoDocumento);
	
    @Component
    @Slf4j
    class Fallback implements EsbClient {

        @Override
        public RiesgosResponse getRiestosCalificacionEndeudamientoCliente(String ente, String moneda, String linea) {
            log.error(String.valueOf(ErrorCode.INTEGRATION_ERROR_RISK));
            return RiesgosResponse.builder().build();
        }

        @Override
        public ElementsClientesConsultaCuentasResponse getClientesConsultaDeCuentas(Long iEnte) {
        	log.error(String.valueOf(ErrorCode.INTEGRATION_ERROR_CUENTAS));
            return ElementsClientesConsultaCuentasResponse.builder().build();
        }

        @Override
        public CondicionesResponse getCondicionesEsb(String canal, String idProducto) {
            log.error(String.valueOf(ErrorCode.INTEGRATION_ERROR_CONDITIONS));
            return CondicionesResponse.builder().build();
        }

        @Override
        public ElementsConsultaDetalleResponse getClientesConsultaDetalleCuentas(String tipoCuenta, Long numeroCuenta, String codigoMoneda) {
            log.error(String.valueOf(ErrorCode.INTEGRATION_ERROR_DETALLES));
            return ElementsConsultaDetalleResponse.builder().build();
        }
        
		@Override
		public ElementsClientesConsultaGeneralResponse getClientesConsultaGeneral(Long icCliente) {
            log.error(String.valueOf(ErrorCode.INTEGRATION_ERROR_CONSULTA_GENERAL));
            return ElementsClientesConsultaGeneralResponse.builder().build();
		}

		@Override
		public ElementsClientesConsultaGeneralResponse getClientesConsultaGeneralPorCuit(String icNumeroIdentificacion, String icTipoDocumento) {
			// TODO Auto-generated method stub
			  return ElementsClientesConsultaGeneralResponse.builder().build();
		}

    }

}
