package ar.com.macro.infraestructure.config;

import ar.com.macro.core.application.repository.LogginRepository;
import feign.Retryer;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import static java.util.concurrent.TimeUnit.SECONDS;

@Configuration
@RequiredArgsConstructor
@EnableFeignClients(basePackages = "ar.com.macro.core.application.outcoming")
public class FeignConfig {

	private final LogginRepository logginRepo;

	@Value("${feign.retry.maxAttempts}")
	private int maxAttemptsApplication;

	@Value("${feign.retry.delay.max}")
	private int maxDelayToRetry;

	@Value("${feign.retry.delay.initial}")
	private int initialDelayToRetry;

	@Bean
	public Retryer retryer() {
		return new Retryer.Default(initialDelayToRetry, SECONDS.toMillis(maxDelayToRetry), maxAttemptsApplication);
	}

	@Bean
	public CustomFeignLogger feignLogger() {
		return new CustomFeignLogger(logginRepo);
	}

}
