package ar.com.macro.infraestructure.log;

import org.apache.commons.lang3.tuple.Pair;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.CodeSignature;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Aspect
@Component
public class LoggingAspect {

	@Around("@annotation(annotation)")
	public Object logExecution(ProceedingJoinPoint joinPoint, Logging annotation) throws Throwable {

		CodeSignature codeSignature = (CodeSignature) joinPoint.getSignature();

		String methodName = codeSignature.getName();

		List<Pair<String, Object>> params = new ArrayList<>();
		for (int i = 0; i < codeSignature.getParameterNames().length; i++) {
			String paramName = codeSignature.getParameterNames()[i];
			Object value = joinPoint.getArgs()[i];
			params.add(Pair.of(paramName, value));
		}

		LogLevel level = forName(annotation.value());
		String className = codeSignature.getDeclaringTypeName();
		Class<?> clazz = Class.forName(className);
		HubLogger logger = HubLogger.load(clazz);

		logger.log(forName(annotation.value()), methodName, params.toArray(new Pair[] {}));

		try {
			Object returnValue = joinPoint.proceed();
			logger.log(level, methodName, returnValue);
			return returnValue;
		}
		catch (Exception e) {
			LogLevel exceptionLevel = forName(annotation.exceptionLevel());
			logger.logException(exceptionLevel, methodName, e);
			throw e;
		}

	}

	private LogLevel forName(String name) {
		String logLevelName = name.trim().toUpperCase();
		// @formatter:off
		return Arrays.asList(LogLevel.values())
				.stream()
				.filter(l -> l.name().equals(logLevelName))
				.findAny()
				.orElse(LogLevel.ERROR);
		// @formatter:on
	}

}
