package ar.com.macro.adapter.controllers;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

import ar.com.macro.adapter.services.AltaAcuerdoCtaCteService;
import ar.com.macro.adapter.services.ProcesoService;
import ar.com.macro.adapter.services.TycsService;
import ar.com.macro.core.application.incoming.request.AltaAcuerdoCtaCteRequest;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.core.application.incoming.request.FirmaRequest;
import ar.com.macro.core.application.incoming.response.AltaAcuerdoCtaCteResponse;
import ar.com.macro.core.application.incoming.response.CancelacionResponse;
import ar.com.macro.core.application.incoming.response.ConsultaTycsResponse;
import java.util.List;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

@ExtendWith(MockitoExtension.class)
class AltaAcuerdoCtaCteControllerTest {

	@InjectMocks
	private AltaAcuerdoCtaCteController altaAcuerdoCtaCteController;
	
	@Mock
	private TycsService tycsService;
	
	@Mock
	private AltaAcuerdoCtaCteService altaAcuerdoCtaCteService;

	@Mock
	private ProcesoService procesoService;
	
	@Test
	@DisplayName("Should return a response with status ok when the request is valid")
	void startValidRequestAltaAcuerdoCtaCte() {
		AltaAcuerdoCtaCteRequest altaAcuerdoCtaCteRequest = AltaAcuerdoCtaCteRequest.builder().build();
		String processInstanceId = "123";
		Mockito.when(altaAcuerdoCtaCteService.iniciar(altaAcuerdoCtaCteRequest)).thenReturn(processInstanceId);

		ResponseEntity<AltaAcuerdoCtaCteResponse> response =
				altaAcuerdoCtaCteController.iniciar(altaAcuerdoCtaCteRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals(HttpStatus.OK.name(), response.getBody().getResultado());
		Assertions.assertEquals(HttpStatus.OK.value(), response.getBody().getCodResultado());
		Assertions.assertEquals(HttpStatus.OK.series().name(), response.getBody().getDescripcion());
		Assertions.assertEquals(processInstanceId, response.getBody().getIdSolicitud());
	}

	@Test
	@DisplayName("Should return a response when call cancellation with status ok when the request is valid")
	void cancelRequestAltaAcuerdoCtaCte() {
		CancelacionRequest cancelationRequest = CancelacionRequest.builder().build();

		Mockito.doAnswer(mocked-> {
			return null;
		}).when(altaAcuerdoCtaCteService).cancelacion(Mockito.any(), Mockito.any());

		ResponseEntity<CancelacionResponse> response =
				altaAcuerdoCtaCteController.cancelacion(cancelationRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals(HttpStatus.OK.series().name(), response.getBody().getDescripcion());
	}

	@Test
	@DisplayName("Should return a response with status ok when the signature is valid")
	void validateSignatureAltaAcuerdoCtaCte() {
		FirmaRequest firmaRequest = FirmaRequest.builder().build();
		Mockito.when(altaAcuerdoCtaCteService.recibirFirma(Mockito.any(), Mockito.any())).thenReturn("123");
		Mockito.when(procesoService.getEstadoDelProceso(any())).thenReturn("estado");

		ResponseEntity<AltaAcuerdoCtaCteResponse> response =
				altaAcuerdoCtaCteController.validarFirma(firmaRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals("123", response.getBody().getIdSolicitud());
		Assertions.assertEquals("estado", response.getBody().getEstadoDelProceso());
	}	
	
	@Test
	@DisplayName("Should return tyc with status ok")
	void getTycsAltaAcuerdoCtaCte() {
		final ResponseEntity<ConsultaTycsResponse> expectedResult = new ResponseEntity<>(
			new ConsultaTycsResponse("OK", 200, "SUCCESSFUL", "idSolicitud", List.of(0)), HttpStatus.OK);
		when(tycsService.getTycsPorIdSolicitud("idSolicitud")).thenReturn(List.of(0));

		final ResponseEntity<ConsultaTycsResponse> result = altaAcuerdoCtaCteController.consultaTycs("idSolicitud");

		assertEquals(expectedResult, result);
	}

}
