package ar.com.macro.adapter.controllers;

import ar.com.macro.adapter.services.DesbordePrestamoService;
import ar.com.macro.adapter.services.ProcesoService;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoIniciarRequest;
import ar.com.macro.core.application.incoming.response.AltaAcuerdoCtaCteResponse;
import ar.com.macro.core.application.incoming.response.DesbordePrestamoDetalleResponse;
import ar.com.macro.core.application.incoming.response.DesbordePrestamoResponse;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import static org.mockito.ArgumentMatchers.any;

import java.util.UUID;

@ExtendWith(MockitoExtension.class)
class DesbordePrestamoControllerTest {

	@Mock
	private DesbordePrestamoService desbordePrestamoService;

	@Mock
	private ProcesoService procesoServicero;

	@InjectMocks
	private DesbordePrestamoController desbordePrestamoController;

	@Test
	@DisplayName("Should return a response with status ok when the request is valid")
	void startWhenRequestIsValid() {
		DesbordePrestamoIniciarRequest desbordePrestamoIniciarRequest = DesbordePrestamoIniciarRequest.builder().build();
		final String PROCESS_INSTANCE_ID = UUID.randomUUID().toString();
		final String ESTADO_DEL_PROCESO = "ESTADO_DEL_PROCESO";

		Mockito.when(desbordePrestamoService.iniciar(desbordePrestamoIniciarRequest)).thenReturn(PROCESS_INSTANCE_ID);
		Mockito.when(procesoServicero.getEstadoDelProceso(Mockito.any())).thenReturn(ESTADO_DEL_PROCESO);

		ResponseEntity<DesbordePrestamoResponse> response =
				desbordePrestamoController.iniciar(desbordePrestamoIniciarRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals(HttpStatus.OK.name(), response.getBody().getResultado());
		Assertions.assertEquals(HttpStatus.OK.value(), response.getBody().getCodResultado());
		Assertions.assertEquals(HttpStatus.OK.series().name(), response.getBody().getDescripcion());
		Assertions.assertEquals(PROCESS_INSTANCE_ID, response.getBody().getIdSolicitud());
	}

	@Test
	@DisplayName("Should throw exception when the request is invalid")
	void startWhenRequestIsValidThenThrowException() {

		DesbordePrestamoIniciarRequest desbordePrestamoIniciarRequest = DesbordePrestamoIniciarRequest.builder().build();
		Mockito.when(desbordePrestamoService.iniciar(desbordePrestamoIniciarRequest)).thenThrow(new RuntimeException());

		Assertions.assertThrows(
				RuntimeException.class,
				() -> {
					desbordePrestamoController.iniciar(desbordePrestamoIniciarRequest);
				});
	}
	
	@Test
	void consultaDetalleWhenRequestIsValid() {
		final String PROCESS_INSTANCE_ID = UUID.randomUUID().toString();
		Mockito.when(desbordePrestamoService.consultaDetalle(PROCESS_INSTANCE_ID)).thenReturn(
				DesbordePrestamoDetalleResponse.builder().idSolicitud(PROCESS_INSTANCE_ID).build()
			);

		DesbordePrestamoDetalleResponse response =
				desbordePrestamoController.consultaDetalle("canal", PROCESS_INSTANCE_ID);

		Assertions.assertEquals(PROCESS_INSTANCE_ID, response.getIdSolicitud());
	}

}