package ar.com.macro.adapter.services;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;

import java.util.UUID;

import org.camunda.bpm.engine.runtime.MessageCorrelationBuilder;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import ar.com.macro.bpm.scope.AltaAcuerdoCtaCteScopeHelper;
import ar.com.macro.core.application.incoming.request.AltaAcuerdoCtaCteRequest;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.infraestructure.exception.BusinessException;

@ExtendWith(MockitoExtension.class)
class AltaAcuerdoCtaCteServiceTest {

	@Mock
	private ProcesoService procesoService;

	@InjectMocks
	private AltaAcuerdoCtaCteService altaAcuerdoCtaCteService;

	@Test
	@DisplayName("Should return the process id when the request is valid")
	void startWhenRequestIsValidThenReturnProcessId() {
		Mockito.when(procesoService.ejecutar(any(), any())).thenReturn("607846f6-7ca0-11ed-b4c1-c6598dc64a4a");
		AltaAcuerdoCtaCteRequest altaAcuerdoCtaCteRequest =
			AltaAcuerdoCtaCteRequest.builder()
				.canal("canal")
				.tipoPersona("tipoPersona")
				.tipoDocumento("DNI")
				.nroDoc(12345678L)
				.idCobis(12345678L)
				.nroCuenta(12345678L)
				.operacion("Primera")
				.plazo(30L)
				.renovacion(false)
				.periodos(1L)
				.monto(100000L)
				.porcenDisminuc(0L)
				.lineaCredito(100000L)
				.tipoLiquidacion("FindeMes")
				.desvioTasa(0L)
				.tipoDesvio("Porcentual")
				.solTipoDoc("DNI")
				.solNumDoc(12345678L)
				.solCaracterInvocado("solCaracterInvocado")
				.solParticipacionSeg(0L)
				.build();

		String processId = this.altaAcuerdoCtaCteService.iniciar(altaAcuerdoCtaCteRequest);

		assertNotNull(processId);
	}

	@Test
	void createMessageCorrelation() {

		String retorno = String.valueOf(this.altaAcuerdoCtaCteService.buscarParametros("1", "1"));

		assertNotNull(retorno);
		assertFalse(retorno.isEmpty());
		assertFalse(retorno.isBlank());
	}

    @Test
    void cancelacionShouldThrowException() {

        CancelacionRequest cancelacionRequest = CancelacionRequest.builder()
                .idCanal("canal123")
                .idSolicitud(UUID.randomUUID().toString())
                .motivoCancelacion("Cancelamento de la Solicitud")
                .cuitFirmante(Long.valueOf(14582))
                .build();

        MessageCorrelationBuilder messageCorrelationBuilder = Mockito.mock(MessageCorrelationBuilder.class);

        Mockito.doReturn(messageCorrelationBuilder).when(this.procesoService).createMessageCorrelation(anyString());
        Mockito.doReturn(true).when(this.procesoService).isActivityInstance(anyString());

		Assertions.assertThrows(BusinessException.class, () -> {
			altaAcuerdoCtaCteService.cancelacion(cancelacionRequest, new AltaAcuerdoCtaCteScopeHelper(null));
		});
    }

    @Test
    void cancelacionShouldSuccess() {
    	
        CancelacionRequest cancelacionRequest = CancelacionRequest.builder()
                .idCanal("canal123")
                .idSolicitud(UUID.randomUUID().toString())
                .motivoCancelacion("Cancelamento de la Solicitud")
                .cuitFirmante(Long.valueOf(14582))
                .build();
    	
        MessageCorrelationBuilder messageCorrelationBuilder = Mockito.mock(MessageCorrelationBuilder.class);
        
        Mockito.doReturn(messageCorrelationBuilder).when(this.procesoService).createMessageCorrelation(anyString());
        Mockito.doReturn(false).when(this.procesoService).isActivityInstance(anyString());

        try {
        	altaAcuerdoCtaCteService.cancelacion(cancelacionRequest, new AltaAcuerdoCtaCteScopeHelper(null));
        } catch (Exception e) {
        	assertEquals(BusinessException.class, e.getClass());
        }
    }
    
}