package ar.com.macro.adapter.services;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.doReturn;

import java.util.Collections;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import ar.com.macro.core.application.outcoming.EsbClient;
import ar.com.macro.core.application.outcoming.response.ClientesConsultaCuentasResponse;
import ar.com.macro.core.application.outcoming.response.ElementsClientesConsultaCuentasResponse;
import ar.com.macro.infraestructure.exception.NotFoundException;

@ExtendWith(MockitoExtension.class)
class ClientesConsultaCuentasServiceTest {

    @InjectMocks
    private ClientesConsultaCuentasService service;

    @Mock
    private EsbClient esbClient;

    @Test
    void getClientsConsultaGeneralReturnSuccessful() {
        EsbClient spy = Mockito.spy(esbClient);
        ElementsClientesConsultaCuentasResponse e = ElementsClientesConsultaCuentasResponse.builder().build();
        ClientesConsultaCuentasResponse c = getClientesConsultaCuentasResponse();
        e.setElements(Collections.singletonList(c));
        assertEquals(c, e.getElements().get(0));
        doReturn(e).when(spy).getClientesConsultaDeCuentas(anyLong());
        assertEquals(e, spy.getClientesConsultaDeCuentas(anyLong()));
        Mockito.doReturn(e)
                .when(this.esbClient)
                .getClientesConsultaDeCuentas(any());
        assertDoesNotThrow(() -> this.service.getClientesConsultaDeCuentas(anyLong()));
    }

    @Test
    void getClientesConsultaGeneralNotFoundException() throws NotFoundException {
    	
        try {
            service.getClientesConsultaDeCuentas(anyLong());
        } catch (NotFoundException e) {
            assertEquals("not_found", e.getMessage());
        }
    }

    private static ClientesConsultaCuentasResponse getClientesConsultaCuentasResponse() {
        ClientesConsultaCuentasResponse c = new ClientesConsultaCuentasResponse();
        c.setOcMoneda("80");
        c.setOcOficina(345);
        c.setOcProducto(3);
        c.setOcTipoCuenta("C");
        c.setOnCuenta(51245325L);
        return c;
    }
}