package ar.com.macro.adapter.services;

import ar.com.macro.core.application.outcoming.EsbClient;
import ar.com.macro.core.application.outcoming.response.ElementsConsultaDetalleResponse;
import ar.com.macro.core.application.outcoming.response.PasivasConsultaDetalleCuentaResponse;
import ar.com.macro.infraestructure.exception.IntegrationException;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.Collections;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.anyLong;

@ExtendWith(MockitoExtension.class)
class PasivasConsultaDetalleCuentaServiceTest {

    @Mock
    private EsbClient esbClient;
    @InjectMocks
    private PasivasConsultaDetalleCuentaService pasivasConsultaDetalleCuentaService;

    @Test
    void getPasivasConsultaDetalleCuenta() {

        Mockito.doReturn(ElementsConsultaDetalleResponse.builder()
                        .elements(Collections.singletonList(new PasivasConsultaDetalleCuentaResponse()))
                        .build())
                .when(this.esbClient)
                .getClientesConsultaDetalleCuentas(anyString(), anyLong(), anyString());

        PasivasConsultaDetalleCuentaResponse response =
                this.pasivasConsultaDetalleCuentaService.consulta("CA", 65465456L, "82");

        assertNotNull(response);
    }

    @Test
    void getPasivasConsultaDetalleCuentaError() {

        Mockito.doReturn(null)
                .when(this.esbClient)
                .getClientesConsultaDetalleCuentas(anyString(), anyLong(), anyString());

        assertThrows(IntegrationException.class,
                () -> this.pasivasConsultaDetalleCuentaService.consulta("CA", 65465456L, "82"));
    }

}