package ar.com.macro.adapter.services;

import static org.junit.Assert.assertTrue;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.anyMap;
import static org.mockito.ArgumentMatchers.anyString;

import ar.com.macro.bpm.scope.DesbordePrestamoScopeHelper;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoActualizarRequest;
import ar.com.macro.infraestructure.exception.BusinessException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import org.camunda.bpm.engine.HistoryService;
import org.camunda.bpm.engine.ProcessEngine;
import org.camunda.bpm.engine.RuntimeService;
import org.camunda.bpm.engine.history.HistoricProcessInstance;
import org.camunda.bpm.engine.history.HistoricProcessInstanceQuery;
import org.camunda.bpm.engine.history.HistoricVariableInstance;
import org.camunda.bpm.engine.history.HistoricVariableInstanceQuery;
import org.camunda.bpm.engine.runtime.ActivityInstance;
import org.camunda.bpm.engine.runtime.MessageCorrelationBuilder;
import org.camunda.bpm.engine.runtime.ProcessInstance;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
class ProcesoServiceTest {

	@Mock
	private RuntimeService runtimeService;
	@Mock
	private ProcessEngine processEngine;
	@InjectMocks
	private ProcesoService procesoService;

	@Test
	@DisplayName("Should not update the variables when the request is invalid")
	void actualizarDesbordePrestamoPorIdWhenRequestIsInvalidThenNotUpdateVariables() {
		DesbordePrestamoActualizarRequest desbordePrestamoActualizarRequest =
			DesbordePrestamoActualizarRequest.builder()
				.idSolicitud("")
				.idUsuario(1)
				.idCanal("")
				.descUsuario("")
				.observaciones("")
				.estadoLead("DESESTIMADO")
				.build();

		procesoService.actualizarDesbordePrestamoPorId(desbordePrestamoActualizarRequest);

		Mockito.verify(runtimeService, Mockito.times(0))
			.setVariable(anyString(), anyString(), anyMap());
	}

	@Test
	@DisplayName("Should update the variables when the request is valid")
	void actualizarDesbordePrestamoPorIdWhenRequestIsValidThenUpdateVariables() {
		DesbordePrestamoActualizarRequest desbordePrestamoActualizarRequest =
			DesbordePrestamoActualizarRequest.builder()
				.idSolicitud(UUID.randomUUID().toString())
				.idUsuario(1)
				.idCanal("canal")
				.descUsuario("descUsuario")
				.estadoLead("DESESTIMADO")
				.observaciones("observaciones")
				.build();

		procesoService.actualizarDesbordePrestamoPorId(desbordePrestamoActualizarRequest);

		Mockito.verify(runtimeService)
			.setVariable(
				desbordePrestamoActualizarRequest.getIdSolicitud(),
				DesbordePrestamoScopeHelper.ID_USUARIO,
				desbordePrestamoActualizarRequest.getIdUsuario());
		Mockito.verify(runtimeService)
			.setVariable(
				desbordePrestamoActualizarRequest.getIdSolicitud(),
				DesbordePrestamoScopeHelper.CANAL,
				desbordePrestamoActualizarRequest.getIdCanal());
		Mockito.verify(runtimeService)
			.setVariable(
				desbordePrestamoActualizarRequest.getIdSolicitud(),
				DesbordePrestamoScopeHelper.DESC_USUARIO,
				desbordePrestamoActualizarRequest.getDescUsuario());
		Mockito.verify(runtimeService)
			.setVariable(
				desbordePrestamoActualizarRequest.getIdSolicitud(),
				DesbordePrestamoScopeHelper.ESTADO_LEAD,
				desbordePrestamoActualizarRequest.getEstadoLead());
		Mockito.verify(runtimeService)
			.setVariable(
				desbordePrestamoActualizarRequest.getIdSolicitud(),
				DesbordePrestamoScopeHelper.OBSERVACIONES,
				desbordePrestamoActualizarRequest.getObservaciones());
	}

	@Test
	@DisplayName("Should return a list of historic process instances when the idsolicitud is null")
	void getProcessWhenIdSolicitudIsNull() {
		String canal = "canal";
		String tipoDoc = "tipoDoc";
		Long nroTributario = 1L;
		String idSolicitud = null;
		String idSolSMO = "idSolSMO";
		HistoricProcessInstance historicProcessInstance =
			Mockito.mock(HistoricProcessInstance.class);
		List<HistoricProcessInstance> historicProcessInstances = new ArrayList<>();
		historicProcessInstances.add(historicProcessInstance);
		HistoryService historyService = Mockito.mock(HistoryService.class);
		Mockito.when(processEngine.getHistoryService()).thenReturn(historyService);
		Mockito.when(historyService.createHistoricProcessInstanceQuery())
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio"))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal)
					.variableValueEquals("RespTipoDoc", tipoDoc))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal)
					.variableValueEquals("RespTipoDoc", tipoDoc)
					.unlimitedList())
			.thenReturn(historicProcessInstances);

		List<HistoricProcessInstance> result =
			procesoService.buscaProcessos(canal, tipoDoc, nroTributario, idSolicitud, idSolSMO);

		assertNotNull(result);
	}

	@Test
	@DisplayName(
		"Should return a list of historic process instances when the idsolicitud is not null")
	void getProcessWhenIdSolicitudIsNotNull() {
		String canal = "canal";
		String tipoDoc = "tipoDoc";
		Long nroTributario = 1L;
		String idSolicitud = "idSolicitud";
		String idSolSMO = "idSolSMO";
		HistoricProcessInstance historicProcessInstance =
			Mockito.mock(HistoricProcessInstance.class);
		List<HistoricProcessInstance> historicProcessInstances = new ArrayList<>();
		historicProcessInstances.add(historicProcessInstance);
		HistoryService historyService = Mockito.mock(HistoryService.class);
		Mockito.when(processEngine.getHistoryService()).thenReturn(historyService);
		Mockito.when(historyService.createHistoricProcessInstanceQuery())
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio"))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal)
					.variableValueEquals("RespTipoDoc", tipoDoc))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal)
					.variableValueEquals("RespTipoDoc", tipoDoc)
					.processInstanceId(idSolicitud))
			.thenReturn(Mockito.mock(HistoricProcessInstanceQuery.class));
		Mockito.when(
				historyService
					.createHistoricProcessInstanceQuery()
					.processDefinitionKey("AltaDeComercio")
					.variableValueEquals("RespNumDoc", nroTributario)
					.variableValueEquals("Canal", canal)
					.variableValueEquals("RespTipoDoc", tipoDoc)
					.processInstanceId(idSolicitud)
					.unlimitedList())
			.thenReturn(historicProcessInstances);

		List<HistoricProcessInstance> result =
			procesoService.buscaProcessos(canal, tipoDoc, nroTributario, idSolicitud, idSolSMO);

		assertNotNull(result);
	}

	@Test
	void createMessageCorrelation() {

		MessageCorrelationBuilder mock = Mockito.mock(MessageCorrelationBuilder.class);
		Mockito.doReturn(mock)
			.when(this.runtimeService)
			.createMessageCorrelation(anyString());

		MessageCorrelationBuilder proceso = this.procesoService.createMessageCorrelation("proceso");

		assertNotNull(proceso);
	}

	@Test
	void startProcess() {
		HistoryService hs = Mockito.mock(HistoryService.class);
		HistoricVariableInstanceQuery hviq = Mockito.mock(HistoricVariableInstanceQuery.class);
		Mockito.doReturn(hviq).when(hs).createHistoricVariableInstanceQuery();
		Mockito.doReturn(hviq).when(hviq).processInstanceId(anyString());
		HistoricVariableInstance hvi = Mockito.mock(HistoricVariableInstance.class);
		Mockito.doReturn("habilitado").when(hvi).getName();
		Mockito.doReturn("true").when(hvi).getValue();
		List<HistoricVariableInstance> variables = Collections.singletonList(hvi);
		Mockito.doReturn(variables).when(hviq).list();

		ProcessInstance pi = Mockito.mock(ProcessInstance.class);
		Mockito.doReturn(UUID.randomUUID().toString()).when(pi).getId();
		Mockito.doReturn(pi).when(this.runtimeService).startProcessInstanceByKey(anyString(), anyMap());
		Mockito.doReturn(UUID.randomUUID().toString()).when(pi).getProcessInstanceId();

		Mockito.doReturn(hs)
			.when(this.processEngine).getHistoryService();

		String proceso = this.procesoService.ejecutar("proceso", Collections.emptyMap());

		assertNotNull(proceso);
		assertFalse(proceso.isEmpty());
		assertFalse(proceso.isBlank());
		Mockito.verify(this.runtimeService, Mockito.times(2))
			.startProcessInstanceByKey(anyString(), anyMap());
	}

	@Test
	void executingInvalidSchedule() {
		HistoryService hs = Mockito.mock(HistoryService.class);
		HistoricVariableInstanceQuery hviq = Mockito.mock(HistoricVariableInstanceQuery.class);
		Mockito.doReturn(hviq).when(hs).createHistoricVariableInstanceQuery();
		Mockito.doReturn(hviq).when(hviq).processInstanceId(anyString());
		HistoricVariableInstance hvi = Mockito.mock(HistoricVariableInstance.class);
		Mockito.doReturn("habilitado").when(hvi).getName();
		Mockito.doReturn("false").when(hvi).getValue();
		List<HistoricVariableInstance> variables = Collections.singletonList(hvi);
		Mockito.doReturn(variables).when(hviq).list();

		ProcessInstance pi = Mockito.mock(ProcessInstance.class);
		Mockito.doReturn(UUID.randomUUID().toString()).when(pi).getId();
		Mockito.doReturn(pi).when(this.runtimeService).startProcessInstanceByKey(anyString(), anyMap());

		Mockito.doReturn(hs)
			.when(this.processEngine).getHistoryService();

		String nombre = "proceso";
		Map<String, Object> variablesProceso = Collections.emptyMap();
		assertThrows(BusinessException.class, () -> this.procesoService.ejecutar(nombre, variablesProceso));
		Mockito.verify(this.runtimeService, Mockito.times(1))
			.startProcessInstanceByKey(anyString(), anyMap());
	}

	@Test
	void getVariable() {

		Mockito.doReturn(new Object()).when(this.runtimeService).getVariable(anyString(), anyString());

		assertNotNull(procesoService.getVariable(anyString(), anyString()));
	}

	@Test
	void isActivityInstanceTrue() {

		ActivityInstance activityInstance = Mockito.mock(ActivityInstance.class);

		Mockito.doReturn(activityInstance).when(this.runtimeService).getActivityInstance(anyString());

		assertTrue(procesoService.isActivityInstance(anyString()));
	}

	@Test
	void isActivityInstanceFalse() {

		Mockito.doReturn(null).when(this.runtimeService).getActivityInstance(anyString());

		assertFalse(procesoService.isActivityInstance(anyString()));
	}
}