package ar.com.macro.adapter.services;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.Bucket;
import com.amazonaws.services.s3.model.ListObjectsV2Result;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import java.io.File;
import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;

@ExtendWith(MockitoExtension.class)
class S3ServiceTest {

    @Mock
    private AmazonS3 amazonS3Client;
    @InjectMocks
    private S3Service s3Service;

    @BeforeEach
    void setup() {
        this.s3Service.setBucketName("bucket_name");
    }

    @Test
    void listarBuckets() {
        Bucket bucket = Mockito.mock(Bucket.class);
        Mockito.doReturn(Collections.singletonList(bucket))
                .when(amazonS3Client)
                .listBuckets();

        List<Bucket> buckets = this.s3Service.listarBuckets();

        assertNotNull(buckets);
        assertFalse(buckets.isEmpty());

    }

    @Test
    void listarArchivosPorBucket() {
        ListObjectsV2Result result = Mockito.mock(ListObjectsV2Result.class);
        Mockito.doReturn(result)
                .when(amazonS3Client)
                .listObjectsV2(anyString());

        S3ObjectSummary s3ObjectSummary = Mockito.mock(S3ObjectSummary.class);

        Mockito.doReturn(Collections.singletonList(s3ObjectSummary))
                .when(result)
                .getObjectSummaries();

        List<S3ObjectSummary> archivos = this.s3Service.listarArchivosPorBucket("bucket");

        assertNotNull(archivos);
        assertFalse(archivos.isEmpty());
    }

    @Test
    void subirArchivo() {

        this.s3Service.subirArchivo("fileKey", new File("path"));

        Mockito.verify(this.amazonS3Client, Mockito.times(1))
                .putObject(anyString(), anyString(), any(File.class));

    }

    @Test
    void suprimirArchivo() {
        this.s3Service.suprimirArchivo("fileKey");

        Mockito.verify(this.amazonS3Client, Mockito.times(1))
                .deleteObject(anyString(), anyString());
    }
}