package ar.com.macro.adapter.services;

import ar.com.macro.adapter.aws.secrets.AwsEsbSecret;
import ar.com.macro.core.application.entities.Token;
import ar.com.macro.core.application.outcoming.TokenEsbClient;
import ar.com.macro.core.application.repository.TokenRepository;
import feign.Request;
import feign.RequestTemplate;
import feign.Response;
import feign.Util;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.mockito.ArgumentMatchers.anyMap;


@ExtendWith(MockitoExtension.class)
class TokenInterceptorServiceTest {

    @Mock
    private TokenEsbClient tokenEsbClient;

    @Mock
    private TokenRepository tokenRepository;
    @Mock
    private  AwsEsbSecret awsEsbSecret;
    @InjectMocks
    private TokenInterceptorService tokenInterceptorService;

    @Test
    void getTokenNull() {

        Mockito.doReturn(Collections.emptyList())
                .when(this.tokenRepository)
                .findAll();

        Map<String, Collection<String>> headers = new HashMap<>();
        headers.put("X-Authorization-token", Collections.singletonList("token"));
        Request request = Request.create(Request.HttpMethod.GET, "/api", Collections.emptyMap(), null, Util.UTF_8,
                new RequestTemplate());
        Response response = Response.builder().request(request).headers(headers).build();
        Mockito.doReturn(response)
                .when(tokenEsbClient)
                .getToken(anyMap());

        Mockito.doReturn("username")
                .when(this.awsEsbSecret).getUsername();
        Mockito.doReturn("password")
                .when(this.awsEsbSecret).getPassword();

        assertNotNull(this.tokenInterceptorService.getToken());

    }

    @Test
    void getTokenExistent() {

        Token token = new Token();
        token.setDate(new Date());
        token.setId(1L);
        Mockito.doReturn(Collections.singletonList(token))
                .when(this.tokenRepository)
                .findAll();

        assertNull(this.tokenInterceptorService.getToken());

    }

    @Test
    void apply() {
        RequestTemplate template = new RequestTemplate();
        template.header("Content-Type", "*/*");
        Map<String, Collection<String>> headers = new HashMap<>();
        headers.put("X-Authorization-token", Collections.singletonList("token"));
        Request request = Request.create(Request.HttpMethod.GET, "/api", Collections.emptyMap(), null, Util.UTF_8,
                new RequestTemplate());
        Response response = Response.builder().request(request).headers(headers).build();
        Mockito.doReturn(response)
                .when(tokenEsbClient)
                .getToken(anyMap());

        Mockito.doReturn(UUID.randomUUID().toString())
                        .when(this.awsEsbSecret).getApplicationId();

        Assertions.assertEquals(template, template.header("Content-Type", "*/*"));

        assertDoesNotThrow(() -> tokenInterceptorService.apply(template));

    }

}