package ar.com.macro.core.application.delegate;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.math.BigDecimal;
import java.util.Collections;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import ar.com.macro.core.application.outcoming.EsbClient;
import ar.com.macro.core.application.outcoming.response.ElementsResponse;
import ar.com.macro.core.application.outcoming.response.RiesgosResponse;

@ExtendWith(MockitoExtension.class)
class ACCVerificarDisponibleDelegateTest {

	@Mock
	private EsbClient esbClient;

	@InjectMocks
	private ACCVerificarDisponibleDelegate accVerificarDisponible;

	@Test
	@DisplayName("Should set totaldisponible to zero when the aperturalineas is empty")
	void shouldExecuteWithSuccess() {
		DelegateExecution delegateExecution = mock(DelegateExecution.class);
		RiesgosResponse riesgosResponse =
			RiesgosResponse.builder()
				.elements(
					Collections.singletonList(
						ElementsResponse.builder()
							.aperturaLineas(Collections.emptyList())
							.build()))
				.build();
		when(esbClient.getRiestosCalificacionEndeudamientoCliente(
			anyString(), anyString(), anyString()))
			.thenReturn(riesgosResponse);
		accVerificarDisponible.execute(delegateExecution);
		verify(delegateExecution, times(1)).setVariable("TotalDisponible", BigDecimal.ZERO);
	}

}