package ar.com.macro.core.application.delegate;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.atLeastOnce;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.verify;

import java.util.Arrays;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import ar.com.macro.adapter.services.ClientesConsultaCuentasService;
import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.core.application.outcoming.response.ClientesConsultaCuentasResponse;
import ar.com.macro.core.application.outcoming.response.ElementsClientesConsultaCuentasResponse;
import ar.com.macro.infraestructure.exception.IntegrationException;

@ExtendWith(MockitoExtension.class)
class AComClientesConsultaCuentasDelegateTest {

    @InjectMocks
    private AComClientesConsultaCuentasDelegate delegate;

    @Mock
    private DelegateExecution execution;
    
    @Mock
    private ClientesConsultaCuentasService clientesConsultaCuentasService;
    
    @Test
    void shouldExecuteWithSuccess() throws Exception {

        Long idCobis = 6856422L;
        Long cuentaAcreditacion = 334509411966251L;

        ElementsClientesConsultaCuentasResponse elements = new ElementsClientesConsultaCuentasResponse();
        
        ClientesConsultaCuentasResponse clientesConsultaCuentasResponse =
        		ClientesConsultaCuentasResponse
        			.builder()
        			.ocMoneda("80")
        			.ocOficina(345)
        			.ocProducto(3)
        			.ocTipoCuenta("C")
        			.onCuenta(cuentaAcreditacion)
        			.build();
        
        elements.setElements(Arrays.asList(clientesConsultaCuentasResponse));
        
        doReturn(elements).when(clientesConsultaCuentasService).getClientesConsultaDeCuentas(idCobis);
        doReturn(idCobis).when(execution).getVariable(AltaComercioScopeHelper.ID_COBIS);
        doReturn(cuentaAcreditacion).when(execution).getVariable(AltaComercioScopeHelper.CUENTA_ACREDITACION);

        delegate.execute(execution);

        verify(clientesConsultaCuentasService, atLeastOnce()).getClientesConsultaDeCuentas(idCobis);

        verify(execution, atLeastOnce()).setVariable(AltaComercioScopeHelper.TIPO_CUENTA, clientesConsultaCuentasResponse.getOcTipoCuenta());
        verify(execution, atLeastOnce()).setVariable(AltaComercioScopeHelper.OFICINA_CUENTA, clientesConsultaCuentasResponse.getOcOficina());
        verify(execution, atLeastOnce()).setVariable(AltaComercioScopeHelper.PRODUCTO_CUENTA, clientesConsultaCuentasResponse.getOcProducto());
        verify(execution, atLeastOnce()).setVariable(AltaComercioScopeHelper.CUENTA_ACREDITACION, clientesConsultaCuentasResponse.getOnCuenta());
        verify(execution, atLeastOnce()).setVariable(AltaComercioScopeHelper.MONEDA_CUENTA, clientesConsultaCuentasResponse.getOcMoneda());
    }
    
    @Test
    void shouldExecuteWithBpmnError() throws Exception {
        Long idCobis = 6856422L;
        Long cuentaAcreditacion = 334509411966251L;

        ElementsClientesConsultaCuentasResponse elements = new ElementsClientesConsultaCuentasResponse();
        
        ClientesConsultaCuentasResponse clientesConsultaCuentasResponse =
        		ClientesConsultaCuentasResponse
        			.builder()
        			.ocMoneda("80")
        			.ocOficina(345)
        			.ocProducto(3)
        			.ocTipoCuenta("C")
        			.onCuenta(1325135454654L)
        			.build();
        
        elements.setElements(Arrays.asList(clientesConsultaCuentasResponse));
        
        doReturn(elements).when(clientesConsultaCuentasService).getClientesConsultaDeCuentas(idCobis);
        doReturn(idCobis).when(execution).getVariable(AltaComercioScopeHelper.ID_COBIS);
        doReturn(cuentaAcreditacion).when(execution).getVariable(AltaComercioScopeHelper.CUENTA_ACREDITACION);

        assertThrows(IntegrationException.class,
                () -> delegate.execute(execution));
    }

}
