package ar.com.macro.core.application.log.filter;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;

import ar.com.macro.core.application.repository.LogginRepository;
import ar.com.macro.infraestructure.exception.IntegrationException;

@ExtendWith(MockitoExtension.class)
class LoggingFilterTest {
    
    @Mock
    private LogginRepository logginRepository;
    
    @InjectMocks
    private LoggingFilter loggingFilter;
    
    @Test
    @DisplayName("Should not save the log when the request is invalid")
    void shouldNotFilterWithPathNotAPI() throws ServletException, IOException {
    	
    	
    	MockHttpServletRequest request = new MockHttpServletRequest();
    	MockHttpServletResponse response = new MockHttpServletResponse();
    	FilterChain filter = Mockito.mock(FilterChain.class);
    	
    	loggingFilter.doFilter(request, response, filter);
    	
    	Mockito.verify(logginRepository, Mockito.times(0)).save(Mockito.any());
    }
    
    @Test
    @DisplayName("Should save the log when the request is valid")
    void shouldFilterWithPathAPI() throws ServletException, IOException {
    	
    	
    	MockHttpServletRequest request = new MockHttpServletRequest();
    	MockHttpServletResponse response = new MockHttpServletResponse();
    	request.setServletPath("/api/altaComercio/iniciar");
    	request.setCharacterEncoding("UTF-8");
    	response.setCharacterEncoding("UTF-8");
    	FilterChain filter = Mockito.mock(FilterChain.class);
    	
    	loggingFilter.doFilter(request, response, filter);
    	
    	Mockito.verify(logginRepository, Mockito.times(1)).save(Mockito.any());
    }
    

    @Test
    @DisplayName("Should return exception")
    void shouldReturnException() throws ServletException, IOException {
    	
    	
    	MockHttpServletRequest request = new MockHttpServletRequest();
    	MockHttpServletResponse response = new MockHttpServletResponse();
    	request.setServletPath("/api/altaComercio/iniciar");
    	request.setCharacterEncoding("UTF-898");
    	response.setCharacterEncoding("UTF-898");
    	FilterChain filter = Mockito.mock(FilterChain.class);
    	
		Assertions.assertThrows(
				IntegrationException.class,
				() -> {
					loggingFilter.doFilter(request, response, filter);;
				});
    }
	
}
